
// -*-c++-*-
//-------------------------------------------------------------------------
//   Copyright 2002-2019 National Technology & Engineering Solutions of
//   Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
//   NTESS, the U.S. Government retains certain rights in this software.
//
//   This file is part of the Xyce(TM) Parallel Electrical Simulator.
//
//   Xyce(TM) is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Xyce(TM) is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Xyce(TM).
//   If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
// Purpose        :
//
// Special Notes  : Generated from verilog file psp102.va with ADMS
//                  interface for Xyce 6.12.0
//                  DO NOT EDIT THIS FILE DIRECTLY!  It may be overwritten!
//
// Creator        : admsXml-2.3.6
//
// Creation Date  : Tue, 11 Jun 2019 12:24:07
//
//-----------------------------------------------------------------------------
#ifndef Xyce_N_DEV_ADMSPSP102VA_h
#define Xyce_N_DEV_ADMSPSP102VA_h


#include <Sacado.hpp>

#include <N_DEV_Configuration.h>
#include <N_DEV_Const.h>
#include <N_DEV_DeviceBlock.h>
#include <N_DEV_DeviceInstance.h>
#include <N_DEV_DeviceModel.h>
#include <N_DEV_MOSFET1.h>


namespace Xyce {
namespace Device {
namespace ADMSPSP102VA {

// This typedef is for our automatic differentiation:
typedef Sacado::Fad::SFad<double,12> AdmsFadType;
typedef Sacado::Fad::SFad<double,1> AdmsSensFadType;

class Model;
class Instance;
class InstanceSensitivity;

#ifdef Xyce_ADMS_SENSITIVITIES
//-----------------------------------------------------------------------------
// Class         : InstanceSensitivity
//
// Purpose       : This class is a functor for sensitivity
//
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class InstanceSensitivity : public baseSensitivity
{
public:
  InstanceSensitivity() :
    baseSensitivity() {};

  virtual ~InstanceSensitivity() {};

  virtual void operator()(
     const ParameterBase &entity,
     const std::string &param,
     std::vector<double> & dfdp,
     std::vector<double> & dqdp,
     std::vector<double> & dbdp,
     std::vector<int> & Findices,
     std::vector<int> & Qindices,
     std::vector<int> & Bindices
                          ) const ;
};

static InstanceSensitivity instSens;


//-----------------------------------------------------------------------------
// Class         : ModelSensitivity
//
// Purpose       : This class is a functor for sensitivity
//
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class ModelSensitivity : public baseSensitivity
{
public:
  ModelSensitivity() :
    baseSensitivity() {};

  virtual ~ModelSensitivity() {};

  virtual void operator()(
     const ParameterBase &entity,
     const std::string &param,
     std::vector<double> & dfdp,
     std::vector<double> & dqdp,
     std::vector<double> & dbdp,
     std::vector<int> & Findices,
     std::vector<int> & Qindices,
     std::vector<int> & Bindices
                          ) const ;
};

static ModelSensitivity modSens;
#endif // Xyce_ADMS_SENSITIVITIES

// general purpose free functions
// thermal voltage at kelvin temperature temp)
template <typename T> static inline T adms_vt(const T temp) {return(CONSTKoverQ*temp);};

// Figure out how to template this shiznit!
//-----------------------------------------------------------------------------


template<typename Tin>
static Tin adms_ternary_op(const bool cond, const Tin &ifTrue, const Tin &ifFalse)
{
  if (cond)
    return ifTrue;
  else
    return ifFalse;
}

template<typename Tin>
static Tin adms_ternary_op(const bool cond, const Tin &ifTrue, const double &ifFalse)
{
  if (cond)
    return ifTrue;
  else
    return Tin(ifFalse);
}

template<typename Tin>
static Tin adms_ternary_op(const bool cond, const double &ifTrue, const Tin &ifFalse)
{
  if (cond)
    return Tin(ifTrue);
  else
    return ifFalse;
}




#ifdef Xyce_ADMS_SENSITIVITIES
//-----------------------------------------------------------------------------
// Free functions used by sensitivity
//
//-----------------------------------------------------------------------------
void evaluateModelEquations(
   std::vector <double> & probeVars,
   // probe constants
   const int admsProbeID_V_NOI_GND,
   const int admsProbeID_V_NOI2_GND,
   const int admsProbeID_V_D_BD,
   const int admsProbeID_V_S_BS,
   const int admsProbeID_V_S_BP,
   const int admsProbeID_V_D_S,
   const int admsProbeID_V_GP_S,
   const int admsProbeID_V_B_BI,
   const int admsProbeID_V_BP_BI,
   const int admsProbeID_V_BD_BI,
   const int admsProbeID_V_BS_BI,
   const int admsProbeID_V_G_GP,
   // node constants
   const int admsNodeID_D,
   const int admsNodeID_G,
   const int admsNodeID_S,
   const int admsNodeID_B,
   const int admsNodeID_NOI,
   const int admsNodeID_NOI2,
   const int admsNodeID_GP,
   const int admsNodeID_BP,
   const int admsNodeID_BI,
   const int admsNodeID_BS,
   const int admsNodeID_BD,
   // instance parameters
   // reals
   AdmsSensFadType & instancePar_L,
   bool instancePar_given_L,
   AdmsSensFadType & instancePar_W,
   bool instancePar_given_W,
   AdmsSensFadType & instancePar_SA,
   bool instancePar_given_SA,
   AdmsSensFadType & instancePar_SB,
   bool instancePar_given_SB,
   AdmsSensFadType & instancePar_SD,
   bool instancePar_given_SD,
   AdmsSensFadType & instancePar_SCA,
   bool instancePar_given_SCA,
   AdmsSensFadType & instancePar_SCB,
   bool instancePar_given_SCB,
   AdmsSensFadType & instancePar_SCC,
   bool instancePar_given_SCC,
   AdmsSensFadType & instancePar_SC,
   bool instancePar_given_SC,
   AdmsSensFadType & instancePar_DELVTO,
   bool instancePar_given_DELVTO,
   AdmsSensFadType & instancePar_FACTUO,
   bool instancePar_given_FACTUO,
   AdmsSensFadType & instancePar_ABSOURCE,
   bool instancePar_given_ABSOURCE,
   AdmsSensFadType & instancePar_LSSOURCE,
   bool instancePar_given_LSSOURCE,
   AdmsSensFadType & instancePar_LGSOURCE,
   bool instancePar_given_LGSOURCE,
   AdmsSensFadType & instancePar_ABDRAIN,
   bool instancePar_given_ABDRAIN,
   AdmsSensFadType & instancePar_LSDRAIN,
   bool instancePar_given_LSDRAIN,
   AdmsSensFadType & instancePar_LGDRAIN,
   bool instancePar_given_LGDRAIN,
   AdmsSensFadType & instancePar_AS,
   bool instancePar_given_AS,
   AdmsSensFadType & instancePar_PS,
   bool instancePar_given_PS,
   AdmsSensFadType & instancePar_AD,
   bool instancePar_given_AD,
   AdmsSensFadType & instancePar_PD,
   bool instancePar_given_PD,
   AdmsSensFadType & instancePar_MULT,
   bool instancePar_given_MULT,
   AdmsSensFadType & instancePar_NF,
   bool instancePar_given_NF,
   AdmsSensFadType & instancePar_NGCON,
   bool instancePar_given_NGCON,
   AdmsSensFadType & instancePar_XGW,
   bool instancePar_given_XGW,
   // instance variables
   // reals
   AdmsSensFadType & instanceVar_VFB_i,
   AdmsSensFadType & instanceVar_VNSUB_i,
   AdmsSensFadType & instanceVar_NSLP_i,
   AdmsSensFadType & instanceVar_DNSUB_i,
   AdmsSensFadType & instanceVar_CF_i,
   AdmsSensFadType & instanceVar_CFB_i,
   AdmsSensFadType & instanceVar_MUE_i,
   AdmsSensFadType & instanceVar_THEMU_i,
   AdmsSensFadType & instanceVar_CS_i,
   AdmsSensFadType & instanceVar_XCOR_i,
   AdmsSensFadType & instanceVar_THER_i,
   AdmsSensFadType & instanceVar_RSB_i,
   AdmsSensFadType & instanceVar_RSG_i,
   AdmsSensFadType & instanceVar_THESAT_i,
   AdmsSensFadType & instanceVar_THESATB_i,
   AdmsSensFadType & instanceVar_THESATG_i,
   AdmsSensFadType & instanceVar_AX_i,
   AdmsSensFadType & instanceVar_ALP_i,
   AdmsSensFadType & instanceVar_ALP1_i,
   AdmsSensFadType & instanceVar_ALP2_i,
   AdmsSensFadType & instanceVar_A1_i,
   AdmsSensFadType & instanceVar_A2_i,
   AdmsSensFadType & instanceVar_A3_i,
   AdmsSensFadType & instanceVar_A4_i,
   AdmsSensFadType & instanceVar_IGINV_i,
   AdmsSensFadType & instanceVar_IGOV_i,
   AdmsSensFadType & instanceVar_IGOVD_i,
   AdmsSensFadType & instanceVar_GC2_i,
   AdmsSensFadType & instanceVar_GC3_i,
   AdmsSensFadType & instanceVar_CHIB_i,
   AdmsSensFadType & instanceVar_AGIDL_i,
   AdmsSensFadType & instanceVar_CGIDL_i,
   AdmsSensFadType & instanceVar_AGIDLD_i,
   AdmsSensFadType & instanceVar_CGIDLD_i,
   AdmsSensFadType & instanceVar_COX_i,
   AdmsSensFadType & instanceVar_CGOV_i,
   AdmsSensFadType & instanceVar_CGOVD_i,
   AdmsSensFadType & instanceVar_CGBOV_i,
   AdmsSensFadType & instanceVar_CFR_i,
   AdmsSensFadType & instanceVar_CFRD_i,
   AdmsSensFadType & instanceVar_NFA_i,
   AdmsSensFadType & instanceVar_NFB_i,
   AdmsSensFadType & instanceVar_NFC_i,
   AdmsSensFadType & instanceVar_EF_i,
   AdmsSensFadType & instanceVar_FNTEXC_i,
   AdmsSensFadType & instanceVar_MULT_i,
   AdmsSensFadType & instanceVar_BET_i,
   AdmsSensFadType & instanceVar_kp,
   AdmsSensFadType & instanceVar_qq,
   AdmsSensFadType & instanceVar_qlim2,
   AdmsSensFadType & instanceVar_E_eff0,
   AdmsSensFadType & instanceVar_eta_mu,
   AdmsSensFadType & instanceVar_BCH,
   AdmsSensFadType & instanceVar_BOV,
   AdmsSensFadType & instanceVar_inv_CHIB,
   AdmsSensFadType & instanceVar_GCQ,
   AdmsSensFadType & instanceVar_Dch,
   AdmsSensFadType & instanceVar_Dov,
   AdmsSensFadType & instanceVar_x1,
   AdmsSensFadType & instanceVar_Vdsat_lim,
   AdmsSensFadType & instanceVar_nt,
   AdmsSensFadType & instanceVar_Cox_over_q,
   AdmsSensFadType & instanceVar_GOV_s,
   AdmsSensFadType & instanceVar_GOV_d,
   AdmsSensFadType & instanceVar_GOV2_s,
   AdmsSensFadType & instanceVar_GOV2_d,
   AdmsSensFadType & instanceVar_BOV_d,
   AdmsSensFadType & instanceVar_xi_ov_s,
   AdmsSensFadType & instanceVar_xi_ov_d,
   AdmsSensFadType & instanceVar_inv_xi_ov_s,
   AdmsSensFadType & instanceVar_inv_xi_ov_d,
   AdmsSensFadType & instanceVar_x_mrg_ov_s,
   AdmsSensFadType & instanceVar_x_mrg_ov_d,
   AdmsSensFadType & instanceVar_inv_xg1_s,
   AdmsSensFadType & instanceVar_inv_xg1_d,
   AdmsSensFadType & instanceVar_AGIDLs,
   AdmsSensFadType & instanceVar_AGIDLDs,
   AdmsSensFadType & instanceVar_BGIDLs,
   AdmsSensFadType & instanceVar_BGIDLDs,
   AdmsSensFadType & instanceVar_phib,
   AdmsSensFadType & instanceVar_sqrt_phib,
   AdmsSensFadType & instanceVar_phix,
   AdmsSensFadType & instanceVar_aphi,
   AdmsSensFadType & instanceVar_bphi,
   AdmsSensFadType & instanceVar_phix1,
   AdmsSensFadType & instanceVar_G_0,
   AdmsSensFadType & instanceVar_phit1,
   AdmsSensFadType & instanceVar_inv_phit1,
   AdmsSensFadType & instanceVar_alpha_b,
   AdmsSensFadType & instanceVar_inv_VP,
   AdmsSensFadType & instanceVar_inv_AX,
   AdmsSensFadType & instanceVar_Sfl_prefac,
   AdmsSensFadType & instanceVar_fac_exc,
   AdmsSensFadType & instanceVar_RG_i,
   AdmsSensFadType & instanceVar_RBULK_i,
   AdmsSensFadType & instanceVar_RWELL_i,
   AdmsSensFadType & instanceVar_RJUNS_i,
   AdmsSensFadType & instanceVar_RJUND_i,
   AdmsSensFadType & instanceVar_ggate,
   AdmsSensFadType & instanceVar_gbulk,
   AdmsSensFadType & instanceVar_gwell,
   AdmsSensFadType & instanceVar_gjund,
   AdmsSensFadType & instanceVar_gjuns,
   AdmsSensFadType & instanceVar_ABSOURCE_i,
   AdmsSensFadType & instanceVar_LSSOURCE_i,
   AdmsSensFadType & instanceVar_LGSOURCE_i,
   AdmsSensFadType & instanceVar_zflagbot_s,
   AdmsSensFadType & instanceVar_zflagsti_s,
   AdmsSensFadType & instanceVar_zflaggat_s,
   AdmsSensFadType & instanceVar_VMAX_s,
   AdmsSensFadType & instanceVar_exp_VMAX_over_phitd_s,
   AdmsSensFadType & instanceVar_vbimin_s,
   AdmsSensFadType & instanceVar_vch_s,
   AdmsSensFadType & instanceVar_vfmin_s,
   AdmsSensFadType & instanceVar_vbbtlim_s,
   AdmsSensFadType & instanceVar_xhighf1_s,
   AdmsSensFadType & instanceVar_expxhf1_s,
   AdmsSensFadType & instanceVar_xhighf2_s,
   AdmsSensFadType & instanceVar_expxhf2_s,
   AdmsSensFadType & instanceVar_xhighr_s,
   AdmsSensFadType & instanceVar_expxhr_s,
   AdmsSensFadType & instanceVar_m0flag_s,
   AdmsSensFadType & instanceVar_ISATFOR1_s,
   AdmsSensFadType & instanceVar_MFOR1_s,
   AdmsSensFadType & instanceVar_ISATFOR2_s,
   AdmsSensFadType & instanceVar_MFOR2_s,
   AdmsSensFadType & instanceVar_ISATREV_s,
   AdmsSensFadType & instanceVar_MREV_s,
   AdmsSensFadType & instanceVar_ABDRAIN_i,
   AdmsSensFadType & instanceVar_LSDRAIN_i,
   AdmsSensFadType & instanceVar_LGDRAIN_i,
   AdmsSensFadType & instanceVar_zflagbot_d,
   AdmsSensFadType & instanceVar_zflagsti_d,
   AdmsSensFadType & instanceVar_zflaggat_d,
   AdmsSensFadType & instanceVar_VMAX_d,
   AdmsSensFadType & instanceVar_exp_VMAX_over_phitd_d,
   AdmsSensFadType & instanceVar_vbimin_d,
   AdmsSensFadType & instanceVar_vch_d,
   AdmsSensFadType & instanceVar_vfmin_d,
   AdmsSensFadType & instanceVar_vbbtlim_d,
   AdmsSensFadType & instanceVar_xhighf1_d,
   AdmsSensFadType & instanceVar_expxhf1_d,
   AdmsSensFadType & instanceVar_xhighf2_d,
   AdmsSensFadType & instanceVar_expxhf2_d,
   AdmsSensFadType & instanceVar_xhighr_d,
   AdmsSensFadType & instanceVar_expxhr_d,
   AdmsSensFadType & instanceVar_m0flag_d,
   AdmsSensFadType & instanceVar_ISATFOR1_d,
   AdmsSensFadType & instanceVar_MFOR1_d,
   AdmsSensFadType & instanceVar_ISATFOR2_d,
   AdmsSensFadType & instanceVar_MFOR2_d,
   AdmsSensFadType & instanceVar_ISATREV_d,
   AdmsSensFadType & instanceVar_MREV_d,
   // model parameters
   // reals
   AdmsSensFadType & modelPar_LEVEL,
   bool modelPar_given_LEVEL,
   AdmsSensFadType & modelPar_TYPE,
   bool modelPar_given_TYPE,
   AdmsSensFadType & modelPar_TR,
   bool modelPar_given_TR,
   AdmsSensFadType & modelPar_SWIGATE,
   bool modelPar_given_SWIGATE,
   AdmsSensFadType & modelPar_SWIMPACT,
   bool modelPar_given_SWIMPACT,
   AdmsSensFadType & modelPar_SWGIDL,
   bool modelPar_given_SWGIDL,
   AdmsSensFadType & modelPar_SWJUNCAP,
   bool modelPar_given_SWJUNCAP,
   AdmsSensFadType & modelPar_SWJUNASYM,
   bool modelPar_given_SWJUNASYM,
   AdmsSensFadType & modelPar_QMC,
   bool modelPar_given_QMC,
   AdmsSensFadType & modelPar_LVARO,
   bool modelPar_given_LVARO,
   AdmsSensFadType & modelPar_LVARL,
   bool modelPar_given_LVARL,
   AdmsSensFadType & modelPar_LVARW,
   bool modelPar_given_LVARW,
   AdmsSensFadType & modelPar_LAP,
   bool modelPar_given_LAP,
   AdmsSensFadType & modelPar_WVARO,
   bool modelPar_given_WVARO,
   AdmsSensFadType & modelPar_WVARL,
   bool modelPar_given_WVARL,
   AdmsSensFadType & modelPar_WVARW,
   bool modelPar_given_WVARW,
   AdmsSensFadType & modelPar_WOT,
   bool modelPar_given_WOT,
   AdmsSensFadType & modelPar_DLQ,
   bool modelPar_given_DLQ,
   AdmsSensFadType & modelPar_DWQ,
   bool modelPar_given_DWQ,
   AdmsSensFadType & modelPar_VFBO,
   bool modelPar_given_VFBO,
   AdmsSensFadType & modelPar_VFBL,
   bool modelPar_given_VFBL,
   AdmsSensFadType & modelPar_VFBW,
   bool modelPar_given_VFBW,
   AdmsSensFadType & modelPar_VFBLW,
   bool modelPar_given_VFBLW,
   AdmsSensFadType & modelPar_STVFBO,
   bool modelPar_given_STVFBO,
   AdmsSensFadType & modelPar_STVFBL,
   bool modelPar_given_STVFBL,
   AdmsSensFadType & modelPar_STVFBW,
   bool modelPar_given_STVFBW,
   AdmsSensFadType & modelPar_STVFBLW,
   bool modelPar_given_STVFBLW,
   AdmsSensFadType & modelPar_TOXO,
   bool modelPar_given_TOXO,
   AdmsSensFadType & modelPar_EPSROXO,
   bool modelPar_given_EPSROXO,
   AdmsSensFadType & modelPar_NSUBO,
   bool modelPar_given_NSUBO,
   AdmsSensFadType & modelPar_NSUBW,
   bool modelPar_given_NSUBW,
   AdmsSensFadType & modelPar_WSEG,
   bool modelPar_given_WSEG,
   AdmsSensFadType & modelPar_NPCK,
   bool modelPar_given_NPCK,
   AdmsSensFadType & modelPar_NPCKW,
   bool modelPar_given_NPCKW,
   AdmsSensFadType & modelPar_WSEGP,
   bool modelPar_given_WSEGP,
   AdmsSensFadType & modelPar_LPCK,
   bool modelPar_given_LPCK,
   AdmsSensFadType & modelPar_LPCKW,
   bool modelPar_given_LPCKW,
   AdmsSensFadType & modelPar_FOL1,
   bool modelPar_given_FOL1,
   AdmsSensFadType & modelPar_FOL2,
   bool modelPar_given_FOL2,
   AdmsSensFadType & modelPar_VNSUBO,
   bool modelPar_given_VNSUBO,
   AdmsSensFadType & modelPar_NSLPO,
   bool modelPar_given_NSLPO,
   AdmsSensFadType & modelPar_DNSUBO,
   bool modelPar_given_DNSUBO,
   AdmsSensFadType & modelPar_DPHIBO,
   bool modelPar_given_DPHIBO,
   AdmsSensFadType & modelPar_DPHIBL,
   bool modelPar_given_DPHIBL,
   AdmsSensFadType & modelPar_DPHIBLEXP,
   bool modelPar_given_DPHIBLEXP,
   AdmsSensFadType & modelPar_DPHIBW,
   bool modelPar_given_DPHIBW,
   AdmsSensFadType & modelPar_DPHIBLW,
   bool modelPar_given_DPHIBLW,
   AdmsSensFadType & modelPar_NPO,
   bool modelPar_given_NPO,
   AdmsSensFadType & modelPar_NPL,
   bool modelPar_given_NPL,
   AdmsSensFadType & modelPar_CTO,
   bool modelPar_given_CTO,
   AdmsSensFadType & modelPar_CTL,
   bool modelPar_given_CTL,
   AdmsSensFadType & modelPar_CTLEXP,
   bool modelPar_given_CTLEXP,
   AdmsSensFadType & modelPar_CTW,
   bool modelPar_given_CTW,
   AdmsSensFadType & modelPar_CTLW,
   bool modelPar_given_CTLW,
   AdmsSensFadType & modelPar_TOXOVO,
   bool modelPar_given_TOXOVO,
   AdmsSensFadType & modelPar_TOXOVDO,
   bool modelPar_given_TOXOVDO,
   AdmsSensFadType & modelPar_LOV,
   bool modelPar_given_LOV,
   AdmsSensFadType & modelPar_LOVD,
   bool modelPar_given_LOVD,
   AdmsSensFadType & modelPar_NOVO,
   bool modelPar_given_NOVO,
   AdmsSensFadType & modelPar_NOVDO,
   bool modelPar_given_NOVDO,
   AdmsSensFadType & modelPar_CFL,
   bool modelPar_given_CFL,
   AdmsSensFadType & modelPar_CFLEXP,
   bool modelPar_given_CFLEXP,
   AdmsSensFadType & modelPar_CFW,
   bool modelPar_given_CFW,
   AdmsSensFadType & modelPar_CFBO,
   bool modelPar_given_CFBO,
   AdmsSensFadType & modelPar_UO,
   bool modelPar_given_UO,
   AdmsSensFadType & modelPar_FBET1,
   bool modelPar_given_FBET1,
   AdmsSensFadType & modelPar_FBET1W,
   bool modelPar_given_FBET1W,
   AdmsSensFadType & modelPar_LP1,
   bool modelPar_given_LP1,
   AdmsSensFadType & modelPar_LP1W,
   bool modelPar_given_LP1W,
   AdmsSensFadType & modelPar_FBET2,
   bool modelPar_given_FBET2,
   AdmsSensFadType & modelPar_LP2,
   bool modelPar_given_LP2,
   AdmsSensFadType & modelPar_BETW1,
   bool modelPar_given_BETW1,
   AdmsSensFadType & modelPar_BETW2,
   bool modelPar_given_BETW2,
   AdmsSensFadType & modelPar_WBET,
   bool modelPar_given_WBET,
   AdmsSensFadType & modelPar_STBETO,
   bool modelPar_given_STBETO,
   AdmsSensFadType & modelPar_STBETL,
   bool modelPar_given_STBETL,
   AdmsSensFadType & modelPar_STBETW,
   bool modelPar_given_STBETW,
   AdmsSensFadType & modelPar_STBETLW,
   bool modelPar_given_STBETLW,
   AdmsSensFadType & modelPar_MUEO,
   bool modelPar_given_MUEO,
   AdmsSensFadType & modelPar_MUEW,
   bool modelPar_given_MUEW,
   AdmsSensFadType & modelPar_STMUEO,
   bool modelPar_given_STMUEO,
   AdmsSensFadType & modelPar_THEMUO,
   bool modelPar_given_THEMUO,
   AdmsSensFadType & modelPar_STTHEMUO,
   bool modelPar_given_STTHEMUO,
   AdmsSensFadType & modelPar_CSO,
   bool modelPar_given_CSO,
   AdmsSensFadType & modelPar_CSL,
   bool modelPar_given_CSL,
   AdmsSensFadType & modelPar_CSLEXP,
   bool modelPar_given_CSLEXP,
   AdmsSensFadType & modelPar_CSW,
   bool modelPar_given_CSW,
   AdmsSensFadType & modelPar_CSLW,
   bool modelPar_given_CSLW,
   AdmsSensFadType & modelPar_STCSO,
   bool modelPar_given_STCSO,
   AdmsSensFadType & modelPar_XCORO,
   bool modelPar_given_XCORO,
   AdmsSensFadType & modelPar_XCORL,
   bool modelPar_given_XCORL,
   AdmsSensFadType & modelPar_XCORW,
   bool modelPar_given_XCORW,
   AdmsSensFadType & modelPar_XCORLW,
   bool modelPar_given_XCORLW,
   AdmsSensFadType & modelPar_STXCORO,
   bool modelPar_given_STXCORO,
   AdmsSensFadType & modelPar_FETAO,
   bool modelPar_given_FETAO,
   AdmsSensFadType & modelPar_RSW1,
   bool modelPar_given_RSW1,
   AdmsSensFadType & modelPar_RSW2,
   bool modelPar_given_RSW2,
   AdmsSensFadType & modelPar_STRSO,
   bool modelPar_given_STRSO,
   AdmsSensFadType & modelPar_RSBO,
   bool modelPar_given_RSBO,
   AdmsSensFadType & modelPar_RSGO,
   bool modelPar_given_RSGO,
   AdmsSensFadType & modelPar_THESATO,
   bool modelPar_given_THESATO,
   AdmsSensFadType & modelPar_THESATL,
   bool modelPar_given_THESATL,
   AdmsSensFadType & modelPar_THESATLEXP,
   bool modelPar_given_THESATLEXP,
   AdmsSensFadType & modelPar_THESATW,
   bool modelPar_given_THESATW,
   AdmsSensFadType & modelPar_THESATLW,
   bool modelPar_given_THESATLW,
   AdmsSensFadType & modelPar_STTHESATO,
   bool modelPar_given_STTHESATO,
   AdmsSensFadType & modelPar_STTHESATL,
   bool modelPar_given_STTHESATL,
   AdmsSensFadType & modelPar_STTHESATW,
   bool modelPar_given_STTHESATW,
   AdmsSensFadType & modelPar_STTHESATLW,
   bool modelPar_given_STTHESATLW,
   AdmsSensFadType & modelPar_THESATBO,
   bool modelPar_given_THESATBO,
   AdmsSensFadType & modelPar_THESATGO,
   bool modelPar_given_THESATGO,
   AdmsSensFadType & modelPar_AXO,
   bool modelPar_given_AXO,
   AdmsSensFadType & modelPar_AXL,
   bool modelPar_given_AXL,
   AdmsSensFadType & modelPar_ALPL,
   bool modelPar_given_ALPL,
   AdmsSensFadType & modelPar_ALPLEXP,
   bool modelPar_given_ALPLEXP,
   AdmsSensFadType & modelPar_ALPW,
   bool modelPar_given_ALPW,
   AdmsSensFadType & modelPar_ALP1L1,
   bool modelPar_given_ALP1L1,
   AdmsSensFadType & modelPar_ALP1LEXP,
   bool modelPar_given_ALP1LEXP,
   AdmsSensFadType & modelPar_ALP1L2,
   bool modelPar_given_ALP1L2,
   AdmsSensFadType & modelPar_ALP1W,
   bool modelPar_given_ALP1W,
   AdmsSensFadType & modelPar_ALP2L1,
   bool modelPar_given_ALP2L1,
   AdmsSensFadType & modelPar_ALP2LEXP,
   bool modelPar_given_ALP2LEXP,
   AdmsSensFadType & modelPar_ALP2L2,
   bool modelPar_given_ALP2L2,
   AdmsSensFadType & modelPar_ALP2W,
   bool modelPar_given_ALP2W,
   AdmsSensFadType & modelPar_VPO,
   bool modelPar_given_VPO,
   AdmsSensFadType & modelPar_A1O,
   bool modelPar_given_A1O,
   AdmsSensFadType & modelPar_A1L,
   bool modelPar_given_A1L,
   AdmsSensFadType & modelPar_A1W,
   bool modelPar_given_A1W,
   AdmsSensFadType & modelPar_A2O,
   bool modelPar_given_A2O,
   AdmsSensFadType & modelPar_STA2O,
   bool modelPar_given_STA2O,
   AdmsSensFadType & modelPar_A3O,
   bool modelPar_given_A3O,
   AdmsSensFadType & modelPar_A3L,
   bool modelPar_given_A3L,
   AdmsSensFadType & modelPar_A3W,
   bool modelPar_given_A3W,
   AdmsSensFadType & modelPar_A4O,
   bool modelPar_given_A4O,
   AdmsSensFadType & modelPar_A4L,
   bool modelPar_given_A4L,
   AdmsSensFadType & modelPar_A4W,
   bool modelPar_given_A4W,
   AdmsSensFadType & modelPar_GCOO,
   bool modelPar_given_GCOO,
   AdmsSensFadType & modelPar_IGINVLW,
   bool modelPar_given_IGINVLW,
   AdmsSensFadType & modelPar_IGOVW,
   bool modelPar_given_IGOVW,
   AdmsSensFadType & modelPar_IGOVDW,
   bool modelPar_given_IGOVDW,
   AdmsSensFadType & modelPar_STIGO,
   bool modelPar_given_STIGO,
   AdmsSensFadType & modelPar_GC2O,
   bool modelPar_given_GC2O,
   AdmsSensFadType & modelPar_GC3O,
   bool modelPar_given_GC3O,
   AdmsSensFadType & modelPar_CHIBO,
   bool modelPar_given_CHIBO,
   AdmsSensFadType & modelPar_AGIDLW,
   bool modelPar_given_AGIDLW,
   AdmsSensFadType & modelPar_AGIDLDW,
   bool modelPar_given_AGIDLDW,
   AdmsSensFadType & modelPar_BGIDLO,
   bool modelPar_given_BGIDLO,
   AdmsSensFadType & modelPar_BGIDLDO,
   bool modelPar_given_BGIDLDO,
   AdmsSensFadType & modelPar_STBGIDLO,
   bool modelPar_given_STBGIDLO,
   AdmsSensFadType & modelPar_STBGIDLDO,
   bool modelPar_given_STBGIDLDO,
   AdmsSensFadType & modelPar_CGIDLO,
   bool modelPar_given_CGIDLO,
   AdmsSensFadType & modelPar_CGIDLDO,
   bool modelPar_given_CGIDLDO,
   AdmsSensFadType & modelPar_CGBOVL,
   bool modelPar_given_CGBOVL,
   AdmsSensFadType & modelPar_CFRW,
   bool modelPar_given_CFRW,
   AdmsSensFadType & modelPar_CFRDW,
   bool modelPar_given_CFRDW,
   AdmsSensFadType & modelPar_FNTO,
   bool modelPar_given_FNTO,
   AdmsSensFadType & modelPar_FNTEXCL,
   bool modelPar_given_FNTEXCL,
   AdmsSensFadType & modelPar_NFALW,
   bool modelPar_given_NFALW,
   AdmsSensFadType & modelPar_NFBLW,
   bool modelPar_given_NFBLW,
   AdmsSensFadType & modelPar_NFCLW,
   bool modelPar_given_NFCLW,
   AdmsSensFadType & modelPar_EFO,
   bool modelPar_given_EFO,
   AdmsSensFadType & modelPar_LINTNOI,
   bool modelPar_given_LINTNOI,
   AdmsSensFadType & modelPar_ALPNOI,
   bool modelPar_given_ALPNOI,
   AdmsSensFadType & modelPar_DTA,
   bool modelPar_given_DTA,
   AdmsSensFadType & modelPar_KVTHOWEO,
   bool modelPar_given_KVTHOWEO,
   AdmsSensFadType & modelPar_KVTHOWEL,
   bool modelPar_given_KVTHOWEL,
   AdmsSensFadType & modelPar_KVTHOWEW,
   bool modelPar_given_KVTHOWEW,
   AdmsSensFadType & modelPar_KVTHOWELW,
   bool modelPar_given_KVTHOWELW,
   AdmsSensFadType & modelPar_KUOWEO,
   bool modelPar_given_KUOWEO,
   AdmsSensFadType & modelPar_KUOWEL,
   bool modelPar_given_KUOWEL,
   AdmsSensFadType & modelPar_KUOWEW,
   bool modelPar_given_KUOWEW,
   AdmsSensFadType & modelPar_KUOWELW,
   bool modelPar_given_KUOWELW,
   AdmsSensFadType & modelPar_LMIN,
   bool modelPar_given_LMIN,
   AdmsSensFadType & modelPar_LMAX,
   bool modelPar_given_LMAX,
   AdmsSensFadType & modelPar_WMIN,
   bool modelPar_given_WMIN,
   AdmsSensFadType & modelPar_WMAX,
   bool modelPar_given_WMAX,
   AdmsSensFadType & modelPar_RGO,
   bool modelPar_given_RGO,
   AdmsSensFadType & modelPar_RBULKO,
   bool modelPar_given_RBULKO,
   AdmsSensFadType & modelPar_RWELLO,
   bool modelPar_given_RWELLO,
   AdmsSensFadType & modelPar_RJUNSO,
   bool modelPar_given_RJUNSO,
   AdmsSensFadType & modelPar_RJUNDO,
   bool modelPar_given_RJUNDO,
   AdmsSensFadType & modelPar_RINT,
   bool modelPar_given_RINT,
   AdmsSensFadType & modelPar_RVPOLY,
   bool modelPar_given_RVPOLY,
   AdmsSensFadType & modelPar_RSHG,
   bool modelPar_given_RSHG,
   AdmsSensFadType & modelPar_DLSIL,
   bool modelPar_given_DLSIL,
   AdmsSensFadType & modelPar_SAREF,
   bool modelPar_given_SAREF,
   AdmsSensFadType & modelPar_SBREF,
   bool modelPar_given_SBREF,
   AdmsSensFadType & modelPar_WLOD,
   bool modelPar_given_WLOD,
   AdmsSensFadType & modelPar_KUO,
   bool modelPar_given_KUO,
   AdmsSensFadType & modelPar_KVSAT,
   bool modelPar_given_KVSAT,
   AdmsSensFadType & modelPar_TKUO,
   bool modelPar_given_TKUO,
   AdmsSensFadType & modelPar_LKUO,
   bool modelPar_given_LKUO,
   AdmsSensFadType & modelPar_WKUO,
   bool modelPar_given_WKUO,
   AdmsSensFadType & modelPar_PKUO,
   bool modelPar_given_PKUO,
   AdmsSensFadType & modelPar_LLODKUO,
   bool modelPar_given_LLODKUO,
   AdmsSensFadType & modelPar_WLODKUO,
   bool modelPar_given_WLODKUO,
   AdmsSensFadType & modelPar_KVTHO,
   bool modelPar_given_KVTHO,
   AdmsSensFadType & modelPar_LKVTHO,
   bool modelPar_given_LKVTHO,
   AdmsSensFadType & modelPar_WKVTHO,
   bool modelPar_given_WKVTHO,
   AdmsSensFadType & modelPar_PKVTHO,
   bool modelPar_given_PKVTHO,
   AdmsSensFadType & modelPar_LLODVTH,
   bool modelPar_given_LLODVTH,
   AdmsSensFadType & modelPar_WLODVTH,
   bool modelPar_given_WLODVTH,
   AdmsSensFadType & modelPar_STETAO,
   bool modelPar_given_STETAO,
   AdmsSensFadType & modelPar_LODETAO,
   bool modelPar_given_LODETAO,
   AdmsSensFadType & modelPar_SCREF,
   bool modelPar_given_SCREF,
   AdmsSensFadType & modelPar_WEB,
   bool modelPar_given_WEB,
   AdmsSensFadType & modelPar_WEC,
   bool modelPar_given_WEC,
   AdmsSensFadType & modelPar_IMAX,
   bool modelPar_given_IMAX,
   AdmsSensFadType & modelPar_TRJ,
   bool modelPar_given_TRJ,
   AdmsSensFadType & modelPar_CJORBOT,
   bool modelPar_given_CJORBOT,
   AdmsSensFadType & modelPar_CJORSTI,
   bool modelPar_given_CJORSTI,
   AdmsSensFadType & modelPar_CJORGAT,
   bool modelPar_given_CJORGAT,
   AdmsSensFadType & modelPar_VBIRBOT,
   bool modelPar_given_VBIRBOT,
   AdmsSensFadType & modelPar_VBIRSTI,
   bool modelPar_given_VBIRSTI,
   AdmsSensFadType & modelPar_VBIRGAT,
   bool modelPar_given_VBIRGAT,
   AdmsSensFadType & modelPar_PBOT,
   bool modelPar_given_PBOT,
   AdmsSensFadType & modelPar_PSTI,
   bool modelPar_given_PSTI,
   AdmsSensFadType & modelPar_PGAT,
   bool modelPar_given_PGAT,
   AdmsSensFadType & modelPar_PHIGBOT,
   bool modelPar_given_PHIGBOT,
   AdmsSensFadType & modelPar_PHIGSTI,
   bool modelPar_given_PHIGSTI,
   AdmsSensFadType & modelPar_PHIGGAT,
   bool modelPar_given_PHIGGAT,
   AdmsSensFadType & modelPar_IDSATRBOT,
   bool modelPar_given_IDSATRBOT,
   AdmsSensFadType & modelPar_IDSATRSTI,
   bool modelPar_given_IDSATRSTI,
   AdmsSensFadType & modelPar_IDSATRGAT,
   bool modelPar_given_IDSATRGAT,
   AdmsSensFadType & modelPar_CSRHBOT,
   bool modelPar_given_CSRHBOT,
   AdmsSensFadType & modelPar_CSRHSTI,
   bool modelPar_given_CSRHSTI,
   AdmsSensFadType & modelPar_CSRHGAT,
   bool modelPar_given_CSRHGAT,
   AdmsSensFadType & modelPar_XJUNSTI,
   bool modelPar_given_XJUNSTI,
   AdmsSensFadType & modelPar_XJUNGAT,
   bool modelPar_given_XJUNGAT,
   AdmsSensFadType & modelPar_CTATBOT,
   bool modelPar_given_CTATBOT,
   AdmsSensFadType & modelPar_CTATSTI,
   bool modelPar_given_CTATSTI,
   AdmsSensFadType & modelPar_CTATGAT,
   bool modelPar_given_CTATGAT,
   AdmsSensFadType & modelPar_MEFFTATBOT,
   bool modelPar_given_MEFFTATBOT,
   AdmsSensFadType & modelPar_MEFFTATSTI,
   bool modelPar_given_MEFFTATSTI,
   AdmsSensFadType & modelPar_MEFFTATGAT,
   bool modelPar_given_MEFFTATGAT,
   AdmsSensFadType & modelPar_CBBTBOT,
   bool modelPar_given_CBBTBOT,
   AdmsSensFadType & modelPar_CBBTSTI,
   bool modelPar_given_CBBTSTI,
   AdmsSensFadType & modelPar_CBBTGAT,
   bool modelPar_given_CBBTGAT,
   AdmsSensFadType & modelPar_FBBTRBOT,
   bool modelPar_given_FBBTRBOT,
   AdmsSensFadType & modelPar_FBBTRSTI,
   bool modelPar_given_FBBTRSTI,
   AdmsSensFadType & modelPar_FBBTRGAT,
   bool modelPar_given_FBBTRGAT,
   AdmsSensFadType & modelPar_STFBBTBOT,
   bool modelPar_given_STFBBTBOT,
   AdmsSensFadType & modelPar_STFBBTSTI,
   bool modelPar_given_STFBBTSTI,
   AdmsSensFadType & modelPar_STFBBTGAT,
   bool modelPar_given_STFBBTGAT,
   AdmsSensFadType & modelPar_VBRBOT,
   bool modelPar_given_VBRBOT,
   AdmsSensFadType & modelPar_VBRSTI,
   bool modelPar_given_VBRSTI,
   AdmsSensFadType & modelPar_VBRGAT,
   bool modelPar_given_VBRGAT,
   AdmsSensFadType & modelPar_PBRBOT,
   bool modelPar_given_PBRBOT,
   AdmsSensFadType & modelPar_PBRSTI,
   bool modelPar_given_PBRSTI,
   AdmsSensFadType & modelPar_PBRGAT,
   bool modelPar_given_PBRGAT,
   AdmsSensFadType & modelPar_CJORBOTD,
   bool modelPar_given_CJORBOTD,
   AdmsSensFadType & modelPar_CJORSTID,
   bool modelPar_given_CJORSTID,
   AdmsSensFadType & modelPar_CJORGATD,
   bool modelPar_given_CJORGATD,
   AdmsSensFadType & modelPar_VBIRBOTD,
   bool modelPar_given_VBIRBOTD,
   AdmsSensFadType & modelPar_VBIRSTID,
   bool modelPar_given_VBIRSTID,
   AdmsSensFadType & modelPar_VBIRGATD,
   bool modelPar_given_VBIRGATD,
   AdmsSensFadType & modelPar_PBOTD,
   bool modelPar_given_PBOTD,
   AdmsSensFadType & modelPar_PSTID,
   bool modelPar_given_PSTID,
   AdmsSensFadType & modelPar_PGATD,
   bool modelPar_given_PGATD,
   AdmsSensFadType & modelPar_PHIGBOTD,
   bool modelPar_given_PHIGBOTD,
   AdmsSensFadType & modelPar_PHIGSTID,
   bool modelPar_given_PHIGSTID,
   AdmsSensFadType & modelPar_PHIGGATD,
   bool modelPar_given_PHIGGATD,
   AdmsSensFadType & modelPar_IDSATRBOTD,
   bool modelPar_given_IDSATRBOTD,
   AdmsSensFadType & modelPar_IDSATRSTID,
   bool modelPar_given_IDSATRSTID,
   AdmsSensFadType & modelPar_IDSATRGATD,
   bool modelPar_given_IDSATRGATD,
   AdmsSensFadType & modelPar_CSRHBOTD,
   bool modelPar_given_CSRHBOTD,
   AdmsSensFadType & modelPar_CSRHSTID,
   bool modelPar_given_CSRHSTID,
   AdmsSensFadType & modelPar_CSRHGATD,
   bool modelPar_given_CSRHGATD,
   AdmsSensFadType & modelPar_XJUNSTID,
   bool modelPar_given_XJUNSTID,
   AdmsSensFadType & modelPar_XJUNGATD,
   bool modelPar_given_XJUNGATD,
   AdmsSensFadType & modelPar_CTATBOTD,
   bool modelPar_given_CTATBOTD,
   AdmsSensFadType & modelPar_CTATSTID,
   bool modelPar_given_CTATSTID,
   AdmsSensFadType & modelPar_CTATGATD,
   bool modelPar_given_CTATGATD,
   AdmsSensFadType & modelPar_MEFFTATBOTD,
   bool modelPar_given_MEFFTATBOTD,
   AdmsSensFadType & modelPar_MEFFTATSTID,
   bool modelPar_given_MEFFTATSTID,
   AdmsSensFadType & modelPar_MEFFTATGATD,
   bool modelPar_given_MEFFTATGATD,
   AdmsSensFadType & modelPar_CBBTBOTD,
   bool modelPar_given_CBBTBOTD,
   AdmsSensFadType & modelPar_CBBTSTID,
   bool modelPar_given_CBBTSTID,
   AdmsSensFadType & modelPar_CBBTGATD,
   bool modelPar_given_CBBTGATD,
   AdmsSensFadType & modelPar_FBBTRBOTD,
   bool modelPar_given_FBBTRBOTD,
   AdmsSensFadType & modelPar_FBBTRSTID,
   bool modelPar_given_FBBTRSTID,
   AdmsSensFadType & modelPar_FBBTRGATD,
   bool modelPar_given_FBBTRGATD,
   AdmsSensFadType & modelPar_STFBBTBOTD,
   bool modelPar_given_STFBBTBOTD,
   AdmsSensFadType & modelPar_STFBBTSTID,
   bool modelPar_given_STFBBTSTID,
   AdmsSensFadType & modelPar_STFBBTGATD,
   bool modelPar_given_STFBBTGATD,
   AdmsSensFadType & modelPar_VBRBOTD,
   bool modelPar_given_VBRBOTD,
   AdmsSensFadType & modelPar_VBRSTID,
   bool modelPar_given_VBRSTID,
   AdmsSensFadType & modelPar_VBRGATD,
   bool modelPar_given_VBRGATD,
   AdmsSensFadType & modelPar_PBRBOTD,
   bool modelPar_given_PBRBOTD,
   AdmsSensFadType & modelPar_PBRSTID,
   bool modelPar_given_PBRSTID,
   AdmsSensFadType & modelPar_PBRGATD,
   bool modelPar_given_PBRGATD,
   AdmsSensFadType & modelPar_SWJUNEXP,
   bool modelPar_given_SWJUNEXP,
   AdmsSensFadType & modelPar_VJUNREF,
   bool modelPar_given_VJUNREF,
   AdmsSensFadType & modelPar_FJUNQ,
   bool modelPar_given_FJUNQ,
   AdmsSensFadType & modelPar_VJUNREFD,
   bool modelPar_given_VJUNREFD,
   AdmsSensFadType & modelPar_FJUNQD,
   bool modelPar_given_FJUNQD// model variables
   ,
   // reals
   AdmsSensFadType & modelVar_NSUBO_i,
   AdmsSensFadType & modelVar_WSEG_i,
   AdmsSensFadType & modelVar_NPCK_i,
   AdmsSensFadType & modelVar_WSEGP_i,
   AdmsSensFadType & modelVar_LPCK_i,
   AdmsSensFadType & modelVar_LOV_i,
   AdmsSensFadType & modelVar_LOVD_i,
   AdmsSensFadType & modelVar_LP1_i,
   AdmsSensFadType & modelVar_LP2_i,
   AdmsSensFadType & modelVar_WBET_i,
   AdmsSensFadType & modelVar_AXL_i,
   AdmsSensFadType & modelVar_ALP1L2_i,
   AdmsSensFadType & modelVar_ALP2L2_i,
   AdmsSensFadType & modelVar_RINT_i,
   AdmsSensFadType & modelVar_RVPOLY_i,
   AdmsSensFadType & modelVar_SAREF_i,
   AdmsSensFadType & modelVar_SBREF_i,
   AdmsSensFadType & modelVar_KVSAT_i,
   AdmsSensFadType & modelVar_LLODKUO_i,
   AdmsSensFadType & modelVar_WLODKUO_i,
   AdmsSensFadType & modelVar_LLODVTH_i,
   AdmsSensFadType & modelVar_WLODVTH_i,
   AdmsSensFadType & modelVar_LODETAO_i,
   AdmsSensFadType & modelVar_SCREF_i,
   AdmsSensFadType & modelVar_WEB_i,
   AdmsSensFadType & modelVar_WEC_i,
   AdmsSensFadType & modelVar_TOX_i,
   AdmsSensFadType & modelVar_EPSROX_i,
   AdmsSensFadType & modelVar_TOXOV_i,
   AdmsSensFadType & modelVar_TOXOVD_i,
   AdmsSensFadType & modelVar_TKD,
   AdmsSensFadType & modelVar_dT,
   AdmsSensFadType & modelVar_rT,
   AdmsSensFadType & modelVar_rTn,
   AdmsSensFadType & modelVar_EPSSI,
   AdmsSensFadType & modelVar_phit,
   AdmsSensFadType & modelVar_inv_phit,
   AdmsSensFadType & modelVar_Eg,
   AdmsSensFadType & modelVar_phibFac,
   AdmsSensFadType & modelVar_nt0,
   AdmsSensFadType & modelVar_IMAX_i,
   AdmsSensFadType & modelVar_VBIRBOT_i,
   AdmsSensFadType & modelVar_VBIRSTI_i,
   AdmsSensFadType & modelVar_VBIRGAT_i,
   AdmsSensFadType & modelVar_PBOT_i,
   AdmsSensFadType & modelVar_PSTI_i,
   AdmsSensFadType & modelVar_PGAT_i,
   AdmsSensFadType & modelVar_CSRHBOT_i,
   AdmsSensFadType & modelVar_CSRHSTI_i,
   AdmsSensFadType & modelVar_CSRHGAT_i,
   AdmsSensFadType & modelVar_CTATBOT_i,
   AdmsSensFadType & modelVar_CTATSTI_i,
   AdmsSensFadType & modelVar_CTATGAT_i,
   AdmsSensFadType & modelVar_CBBTBOT_i,
   AdmsSensFadType & modelVar_CBBTSTI_i,
   AdmsSensFadType & modelVar_CBBTGAT_i,
   AdmsSensFadType & modelVar_VBRBOT_i,
   AdmsSensFadType & modelVar_VBRSTI_i,
   AdmsSensFadType & modelVar_VBRGAT_i,
   AdmsSensFadType & modelVar_PBRBOT_i,
   AdmsSensFadType & modelVar_PBRSTI_i,
   AdmsSensFadType & modelVar_PBRGAT_i,
   AdmsSensFadType & modelVar_SWJUNEXP_i,
   AdmsSensFadType & modelVar_VJUNREF_i,
   AdmsSensFadType & modelVar_FJUNQ_i,
   AdmsSensFadType & modelVar_phitr,
   AdmsSensFadType & modelVar_phitd,
   AdmsSensFadType & modelVar_phitdinv,
   AdmsSensFadType & modelVar_perfc,
   AdmsSensFadType & modelVar_berfc,
   AdmsSensFadType & modelVar_cerfc,
   AdmsSensFadType & modelVar_ftdbot,
   AdmsSensFadType & modelVar_ftdsti,
   AdmsSensFadType & modelVar_ftdgat,
   AdmsSensFadType & modelVar_idsatbot,
   AdmsSensFadType & modelVar_idsatsti,
   AdmsSensFadType & modelVar_idsatgat,
   AdmsSensFadType & modelVar_vbibot,
   AdmsSensFadType & modelVar_vbisti,
   AdmsSensFadType & modelVar_vbigat,
   AdmsSensFadType & modelVar_vbiinvbot,
   AdmsSensFadType & modelVar_vbiinvsti,
   AdmsSensFadType & modelVar_vbiinvgat,
   AdmsSensFadType & modelVar_one_minus_PBOT,
   AdmsSensFadType & modelVar_one_minus_PSTI,
   AdmsSensFadType & modelVar_one_minus_PGAT,
   AdmsSensFadType & modelVar_one_over_one_minus_PBOT,
   AdmsSensFadType & modelVar_one_over_one_minus_PSTI,
   AdmsSensFadType & modelVar_one_over_one_minus_PGAT,
   AdmsSensFadType & modelVar_cjobot,
   AdmsSensFadType & modelVar_cjosti,
   AdmsSensFadType & modelVar_cjogat,
   AdmsSensFadType & modelVar_qprefbot,
   AdmsSensFadType & modelVar_qprefsti,
   AdmsSensFadType & modelVar_qprefgat,
   AdmsSensFadType & modelVar_qpref2bot,
   AdmsSensFadType & modelVar_qpref2sti,
   AdmsSensFadType & modelVar_qpref2gat,
   AdmsSensFadType & modelVar_wdepnulrbot,
   AdmsSensFadType & modelVar_wdepnulrsti,
   AdmsSensFadType & modelVar_wdepnulrgat,
   AdmsSensFadType & modelVar_wdepnulrinvbot,
   AdmsSensFadType & modelVar_wdepnulrinvsti,
   AdmsSensFadType & modelVar_wdepnulrinvgat,
   AdmsSensFadType & modelVar_VBIRBOTinv,
   AdmsSensFadType & modelVar_VBIRSTIinv,
   AdmsSensFadType & modelVar_VBIRGATinv,
   AdmsSensFadType & modelVar_atatbot,
   AdmsSensFadType & modelVar_atatsti,
   AdmsSensFadType & modelVar_atatgat,
   AdmsSensFadType & modelVar_btatpartbot,
   AdmsSensFadType & modelVar_btatpartsti,
   AdmsSensFadType & modelVar_btatpartgat,
   AdmsSensFadType & modelVar_fbbtbot,
   AdmsSensFadType & modelVar_fbbtsti,
   AdmsSensFadType & modelVar_fbbtgat,
   AdmsSensFadType & modelVar_fstopbot,
   AdmsSensFadType & modelVar_fstopsti,
   AdmsSensFadType & modelVar_fstopgat,
   AdmsSensFadType & modelVar_VBRinvbot,
   AdmsSensFadType & modelVar_VBRinvsti,
   AdmsSensFadType & modelVar_VBRinvgat,
   AdmsSensFadType & modelVar_slopebot,
   AdmsSensFadType & modelVar_slopesti,
   AdmsSensFadType & modelVar_slopegat,
   AdmsSensFadType & modelVar_VBIRBOTD_i,
   AdmsSensFadType & modelVar_VBIRSTID_i,
   AdmsSensFadType & modelVar_VBIRGATD_i,
   AdmsSensFadType & modelVar_PBOTD_i,
   AdmsSensFadType & modelVar_PSTID_i,
   AdmsSensFadType & modelVar_PGATD_i,
   AdmsSensFadType & modelVar_CSRHBOTD_i,
   AdmsSensFadType & modelVar_CSRHSTID_i,
   AdmsSensFadType & modelVar_CSRHGATD_i,
   AdmsSensFadType & modelVar_CTATBOTD_i,
   AdmsSensFadType & modelVar_CTATSTID_i,
   AdmsSensFadType & modelVar_CTATGATD_i,
   AdmsSensFadType & modelVar_CBBTBOTD_i,
   AdmsSensFadType & modelVar_CBBTSTID_i,
   AdmsSensFadType & modelVar_CBBTGATD_i,
   AdmsSensFadType & modelVar_VBRBOTD_i,
   AdmsSensFadType & modelVar_VBRSTID_i,
   AdmsSensFadType & modelVar_VBRGATD_i,
   AdmsSensFadType & modelVar_PBRBOTD_i,
   AdmsSensFadType & modelVar_PBRSTID_i,
   AdmsSensFadType & modelVar_PBRGATD_i,
   AdmsSensFadType & modelVar_VJUNREFD_i,
   AdmsSensFadType & modelVar_FJUNQD_i,
   AdmsSensFadType & modelVar_ftdbot_d,
   AdmsSensFadType & modelVar_ftdsti_d,
   AdmsSensFadType & modelVar_ftdgat_d,
   AdmsSensFadType & modelVar_idsatbot_d,
   AdmsSensFadType & modelVar_idsatsti_d,
   AdmsSensFadType & modelVar_idsatgat_d,
   AdmsSensFadType & modelVar_vbibot_d,
   AdmsSensFadType & modelVar_vbisti_d,
   AdmsSensFadType & modelVar_vbigat_d,
   AdmsSensFadType & modelVar_vbiinvbot_d,
   AdmsSensFadType & modelVar_vbiinvsti_d,
   AdmsSensFadType & modelVar_vbiinvgat_d,
   AdmsSensFadType & modelVar_one_minus_PBOT_d,
   AdmsSensFadType & modelVar_one_minus_PSTI_d,
   AdmsSensFadType & modelVar_one_minus_PGAT_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PBOT_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PSTI_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PGAT_d,
   AdmsSensFadType & modelVar_cjobot_d,
   AdmsSensFadType & modelVar_cjosti_d,
   AdmsSensFadType & modelVar_cjogat_d,
   AdmsSensFadType & modelVar_qprefbot_d,
   AdmsSensFadType & modelVar_qprefsti_d,
   AdmsSensFadType & modelVar_qprefgat_d,
   AdmsSensFadType & modelVar_qpref2bot_d,
   AdmsSensFadType & modelVar_qpref2sti_d,
   AdmsSensFadType & modelVar_qpref2gat_d,
   AdmsSensFadType & modelVar_wdepnulrbot_d,
   AdmsSensFadType & modelVar_wdepnulrsti_d,
   AdmsSensFadType & modelVar_wdepnulrgat_d,
   AdmsSensFadType & modelVar_wdepnulrinvbot_d,
   AdmsSensFadType & modelVar_wdepnulrinvsti_d,
   AdmsSensFadType & modelVar_wdepnulrinvgat_d,
   AdmsSensFadType & modelVar_VBIRBOTinv_d,
   AdmsSensFadType & modelVar_VBIRSTIinv_d,
   AdmsSensFadType & modelVar_VBIRGATinv_d,
   AdmsSensFadType & modelVar_atatbot_d,
   AdmsSensFadType & modelVar_atatsti_d,
   AdmsSensFadType & modelVar_atatgat_d,
   AdmsSensFadType & modelVar_btatpartbot_d,
   AdmsSensFadType & modelVar_btatpartsti_d,
   AdmsSensFadType & modelVar_btatpartgat_d,
   AdmsSensFadType & modelVar_fbbtbot_d,
   AdmsSensFadType & modelVar_fbbtsti_d,
   AdmsSensFadType & modelVar_fbbtgat_d,
   AdmsSensFadType & modelVar_fstopbot_d,
   AdmsSensFadType & modelVar_fstopsti_d,
   AdmsSensFadType & modelVar_fstopgat_d,
   AdmsSensFadType & modelVar_VBRinvbot_d,
   AdmsSensFadType & modelVar_VBRinvsti_d,
   AdmsSensFadType & modelVar_VBRinvgat_d,
   AdmsSensFadType & modelVar_slopebot_d,
   AdmsSensFadType & modelVar_slopesti_d,
   AdmsSensFadType & modelVar_slopegat_d,
   // non-reals
   int modelVar_CHNL_TYPE,
   // basic variables
   double admsTemperature, double adms_vt_nom, double ADMSgmin_arg, std::vector <AdmsSensFadType> & staticContributions, std::vector <AdmsSensFadType> & dynamicContributions, const Instance & theInstance);

void evaluateInitialInstance(
   // instance parameters
   // reals
   AdmsSensFadType & instancePar_L,
   bool instancePar_given_L,
   AdmsSensFadType & instancePar_W,
   bool instancePar_given_W,
   AdmsSensFadType & instancePar_SA,
   bool instancePar_given_SA,
   AdmsSensFadType & instancePar_SB,
   bool instancePar_given_SB,
   AdmsSensFadType & instancePar_SD,
   bool instancePar_given_SD,
   AdmsSensFadType & instancePar_SCA,
   bool instancePar_given_SCA,
   AdmsSensFadType & instancePar_SCB,
   bool instancePar_given_SCB,
   AdmsSensFadType & instancePar_SCC,
   bool instancePar_given_SCC,
   AdmsSensFadType & instancePar_SC,
   bool instancePar_given_SC,
   AdmsSensFadType & instancePar_DELVTO,
   bool instancePar_given_DELVTO,
   AdmsSensFadType & instancePar_FACTUO,
   bool instancePar_given_FACTUO,
   AdmsSensFadType & instancePar_ABSOURCE,
   bool instancePar_given_ABSOURCE,
   AdmsSensFadType & instancePar_LSSOURCE,
   bool instancePar_given_LSSOURCE,
   AdmsSensFadType & instancePar_LGSOURCE,
   bool instancePar_given_LGSOURCE,
   AdmsSensFadType & instancePar_ABDRAIN,
   bool instancePar_given_ABDRAIN,
   AdmsSensFadType & instancePar_LSDRAIN,
   bool instancePar_given_LSDRAIN,
   AdmsSensFadType & instancePar_LGDRAIN,
   bool instancePar_given_LGDRAIN,
   AdmsSensFadType & instancePar_AS,
   bool instancePar_given_AS,
   AdmsSensFadType & instancePar_PS,
   bool instancePar_given_PS,
   AdmsSensFadType & instancePar_AD,
   bool instancePar_given_AD,
   AdmsSensFadType & instancePar_PD,
   bool instancePar_given_PD,
   AdmsSensFadType & instancePar_MULT,
   bool instancePar_given_MULT,
   AdmsSensFadType & instancePar_NF,
   bool instancePar_given_NF,
   AdmsSensFadType & instancePar_NGCON,
   bool instancePar_given_NGCON,
   AdmsSensFadType & instancePar_XGW,
   bool instancePar_given_XGW,
   // instance variables
   // reals
   AdmsSensFadType & instanceVar_VFB_i,
   AdmsSensFadType & instanceVar_VNSUB_i,
   AdmsSensFadType & instanceVar_NSLP_i,
   AdmsSensFadType & instanceVar_DNSUB_i,
   AdmsSensFadType & instanceVar_CF_i,
   AdmsSensFadType & instanceVar_CFB_i,
   AdmsSensFadType & instanceVar_MUE_i,
   AdmsSensFadType & instanceVar_THEMU_i,
   AdmsSensFadType & instanceVar_CS_i,
   AdmsSensFadType & instanceVar_XCOR_i,
   AdmsSensFadType & instanceVar_THER_i,
   AdmsSensFadType & instanceVar_RSB_i,
   AdmsSensFadType & instanceVar_RSG_i,
   AdmsSensFadType & instanceVar_THESAT_i,
   AdmsSensFadType & instanceVar_THESATB_i,
   AdmsSensFadType & instanceVar_THESATG_i,
   AdmsSensFadType & instanceVar_AX_i,
   AdmsSensFadType & instanceVar_ALP_i,
   AdmsSensFadType & instanceVar_ALP1_i,
   AdmsSensFadType & instanceVar_ALP2_i,
   AdmsSensFadType & instanceVar_A1_i,
   AdmsSensFadType & instanceVar_A2_i,
   AdmsSensFadType & instanceVar_A3_i,
   AdmsSensFadType & instanceVar_A4_i,
   AdmsSensFadType & instanceVar_IGINV_i,
   AdmsSensFadType & instanceVar_IGOV_i,
   AdmsSensFadType & instanceVar_IGOVD_i,
   AdmsSensFadType & instanceVar_GC2_i,
   AdmsSensFadType & instanceVar_GC3_i,
   AdmsSensFadType & instanceVar_CHIB_i,
   AdmsSensFadType & instanceVar_AGIDL_i,
   AdmsSensFadType & instanceVar_CGIDL_i,
   AdmsSensFadType & instanceVar_AGIDLD_i,
   AdmsSensFadType & instanceVar_CGIDLD_i,
   AdmsSensFadType & instanceVar_COX_i,
   AdmsSensFadType & instanceVar_CGOV_i,
   AdmsSensFadType & instanceVar_CGOVD_i,
   AdmsSensFadType & instanceVar_CGBOV_i,
   AdmsSensFadType & instanceVar_CFR_i,
   AdmsSensFadType & instanceVar_CFRD_i,
   AdmsSensFadType & instanceVar_NFA_i,
   AdmsSensFadType & instanceVar_NFB_i,
   AdmsSensFadType & instanceVar_NFC_i,
   AdmsSensFadType & instanceVar_EF_i,
   AdmsSensFadType & instanceVar_FNTEXC_i,
   AdmsSensFadType & instanceVar_MULT_i,
   AdmsSensFadType & instanceVar_BET_i,
   AdmsSensFadType & instanceVar_kp,
   AdmsSensFadType & instanceVar_qq,
   AdmsSensFadType & instanceVar_qlim2,
   AdmsSensFadType & instanceVar_E_eff0,
   AdmsSensFadType & instanceVar_eta_mu,
   AdmsSensFadType & instanceVar_BCH,
   AdmsSensFadType & instanceVar_BOV,
   AdmsSensFadType & instanceVar_inv_CHIB,
   AdmsSensFadType & instanceVar_GCQ,
   AdmsSensFadType & instanceVar_Dch,
   AdmsSensFadType & instanceVar_Dov,
   AdmsSensFadType & instanceVar_x1,
   AdmsSensFadType & instanceVar_Vdsat_lim,
   AdmsSensFadType & instanceVar_nt,
   AdmsSensFadType & instanceVar_Cox_over_q,
   AdmsSensFadType & instanceVar_GOV_s,
   AdmsSensFadType & instanceVar_GOV_d,
   AdmsSensFadType & instanceVar_GOV2_s,
   AdmsSensFadType & instanceVar_GOV2_d,
   AdmsSensFadType & instanceVar_BOV_d,
   AdmsSensFadType & instanceVar_xi_ov_s,
   AdmsSensFadType & instanceVar_xi_ov_d,
   AdmsSensFadType & instanceVar_inv_xi_ov_s,
   AdmsSensFadType & instanceVar_inv_xi_ov_d,
   AdmsSensFadType & instanceVar_x_mrg_ov_s,
   AdmsSensFadType & instanceVar_x_mrg_ov_d,
   AdmsSensFadType & instanceVar_inv_xg1_s,
   AdmsSensFadType & instanceVar_inv_xg1_d,
   AdmsSensFadType & instanceVar_AGIDLs,
   AdmsSensFadType & instanceVar_AGIDLDs,
   AdmsSensFadType & instanceVar_BGIDLs,
   AdmsSensFadType & instanceVar_BGIDLDs,
   AdmsSensFadType & instanceVar_phib,
   AdmsSensFadType & instanceVar_sqrt_phib,
   AdmsSensFadType & instanceVar_phix,
   AdmsSensFadType & instanceVar_aphi,
   AdmsSensFadType & instanceVar_bphi,
   AdmsSensFadType & instanceVar_phix1,
   AdmsSensFadType & instanceVar_G_0,
   AdmsSensFadType & instanceVar_phit1,
   AdmsSensFadType & instanceVar_inv_phit1,
   AdmsSensFadType & instanceVar_alpha_b,
   AdmsSensFadType & instanceVar_inv_VP,
   AdmsSensFadType & instanceVar_inv_AX,
   AdmsSensFadType & instanceVar_Sfl_prefac,
   AdmsSensFadType & instanceVar_fac_exc,
   AdmsSensFadType & instanceVar_RG_i,
   AdmsSensFadType & instanceVar_RBULK_i,
   AdmsSensFadType & instanceVar_RWELL_i,
   AdmsSensFadType & instanceVar_RJUNS_i,
   AdmsSensFadType & instanceVar_RJUND_i,
   AdmsSensFadType & instanceVar_ggate,
   AdmsSensFadType & instanceVar_gbulk,
   AdmsSensFadType & instanceVar_gwell,
   AdmsSensFadType & instanceVar_gjund,
   AdmsSensFadType & instanceVar_gjuns,
   AdmsSensFadType & instanceVar_ABSOURCE_i,
   AdmsSensFadType & instanceVar_LSSOURCE_i,
   AdmsSensFadType & instanceVar_LGSOURCE_i,
   AdmsSensFadType & instanceVar_zflagbot_s,
   AdmsSensFadType & instanceVar_zflagsti_s,
   AdmsSensFadType & instanceVar_zflaggat_s,
   AdmsSensFadType & instanceVar_VMAX_s,
   AdmsSensFadType & instanceVar_exp_VMAX_over_phitd_s,
   AdmsSensFadType & instanceVar_vbimin_s,
   AdmsSensFadType & instanceVar_vch_s,
   AdmsSensFadType & instanceVar_vfmin_s,
   AdmsSensFadType & instanceVar_vbbtlim_s,
   AdmsSensFadType & instanceVar_xhighf1_s,
   AdmsSensFadType & instanceVar_expxhf1_s,
   AdmsSensFadType & instanceVar_xhighf2_s,
   AdmsSensFadType & instanceVar_expxhf2_s,
   AdmsSensFadType & instanceVar_xhighr_s,
   AdmsSensFadType & instanceVar_expxhr_s,
   AdmsSensFadType & instanceVar_m0flag_s,
   AdmsSensFadType & instanceVar_ISATFOR1_s,
   AdmsSensFadType & instanceVar_MFOR1_s,
   AdmsSensFadType & instanceVar_ISATFOR2_s,
   AdmsSensFadType & instanceVar_MFOR2_s,
   AdmsSensFadType & instanceVar_ISATREV_s,
   AdmsSensFadType & instanceVar_MREV_s,
   AdmsSensFadType & instanceVar_ABDRAIN_i,
   AdmsSensFadType & instanceVar_LSDRAIN_i,
   AdmsSensFadType & instanceVar_LGDRAIN_i,
   AdmsSensFadType & instanceVar_zflagbot_d,
   AdmsSensFadType & instanceVar_zflagsti_d,
   AdmsSensFadType & instanceVar_zflaggat_d,
   AdmsSensFadType & instanceVar_VMAX_d,
   AdmsSensFadType & instanceVar_exp_VMAX_over_phitd_d,
   AdmsSensFadType & instanceVar_vbimin_d,
   AdmsSensFadType & instanceVar_vch_d,
   AdmsSensFadType & instanceVar_vfmin_d,
   AdmsSensFadType & instanceVar_vbbtlim_d,
   AdmsSensFadType & instanceVar_xhighf1_d,
   AdmsSensFadType & instanceVar_expxhf1_d,
   AdmsSensFadType & instanceVar_xhighf2_d,
   AdmsSensFadType & instanceVar_expxhf2_d,
   AdmsSensFadType & instanceVar_xhighr_d,
   AdmsSensFadType & instanceVar_expxhr_d,
   AdmsSensFadType & instanceVar_m0flag_d,
   AdmsSensFadType & instanceVar_ISATFOR1_d,
   AdmsSensFadType & instanceVar_MFOR1_d,
   AdmsSensFadType & instanceVar_ISATFOR2_d,
   AdmsSensFadType & instanceVar_MFOR2_d,
   AdmsSensFadType & instanceVar_ISATREV_d,
   AdmsSensFadType & instanceVar_MREV_d,
   // model parameters
   // reals
   AdmsSensFadType & modelPar_LEVEL,
   bool modelPar_given_LEVEL,
   AdmsSensFadType & modelPar_TYPE,
   bool modelPar_given_TYPE,
   AdmsSensFadType & modelPar_TR,
   bool modelPar_given_TR,
   AdmsSensFadType & modelPar_SWIGATE,
   bool modelPar_given_SWIGATE,
   AdmsSensFadType & modelPar_SWIMPACT,
   bool modelPar_given_SWIMPACT,
   AdmsSensFadType & modelPar_SWGIDL,
   bool modelPar_given_SWGIDL,
   AdmsSensFadType & modelPar_SWJUNCAP,
   bool modelPar_given_SWJUNCAP,
   AdmsSensFadType & modelPar_SWJUNASYM,
   bool modelPar_given_SWJUNASYM,
   AdmsSensFadType & modelPar_QMC,
   bool modelPar_given_QMC,
   AdmsSensFadType & modelPar_LVARO,
   bool modelPar_given_LVARO,
   AdmsSensFadType & modelPar_LVARL,
   bool modelPar_given_LVARL,
   AdmsSensFadType & modelPar_LVARW,
   bool modelPar_given_LVARW,
   AdmsSensFadType & modelPar_LAP,
   bool modelPar_given_LAP,
   AdmsSensFadType & modelPar_WVARO,
   bool modelPar_given_WVARO,
   AdmsSensFadType & modelPar_WVARL,
   bool modelPar_given_WVARL,
   AdmsSensFadType & modelPar_WVARW,
   bool modelPar_given_WVARW,
   AdmsSensFadType & modelPar_WOT,
   bool modelPar_given_WOT,
   AdmsSensFadType & modelPar_DLQ,
   bool modelPar_given_DLQ,
   AdmsSensFadType & modelPar_DWQ,
   bool modelPar_given_DWQ,
   AdmsSensFadType & modelPar_VFBO,
   bool modelPar_given_VFBO,
   AdmsSensFadType & modelPar_VFBL,
   bool modelPar_given_VFBL,
   AdmsSensFadType & modelPar_VFBW,
   bool modelPar_given_VFBW,
   AdmsSensFadType & modelPar_VFBLW,
   bool modelPar_given_VFBLW,
   AdmsSensFadType & modelPar_STVFBO,
   bool modelPar_given_STVFBO,
   AdmsSensFadType & modelPar_STVFBL,
   bool modelPar_given_STVFBL,
   AdmsSensFadType & modelPar_STVFBW,
   bool modelPar_given_STVFBW,
   AdmsSensFadType & modelPar_STVFBLW,
   bool modelPar_given_STVFBLW,
   AdmsSensFadType & modelPar_TOXO,
   bool modelPar_given_TOXO,
   AdmsSensFadType & modelPar_EPSROXO,
   bool modelPar_given_EPSROXO,
   AdmsSensFadType & modelPar_NSUBO,
   bool modelPar_given_NSUBO,
   AdmsSensFadType & modelPar_NSUBW,
   bool modelPar_given_NSUBW,
   AdmsSensFadType & modelPar_WSEG,
   bool modelPar_given_WSEG,
   AdmsSensFadType & modelPar_NPCK,
   bool modelPar_given_NPCK,
   AdmsSensFadType & modelPar_NPCKW,
   bool modelPar_given_NPCKW,
   AdmsSensFadType & modelPar_WSEGP,
   bool modelPar_given_WSEGP,
   AdmsSensFadType & modelPar_LPCK,
   bool modelPar_given_LPCK,
   AdmsSensFadType & modelPar_LPCKW,
   bool modelPar_given_LPCKW,
   AdmsSensFadType & modelPar_FOL1,
   bool modelPar_given_FOL1,
   AdmsSensFadType & modelPar_FOL2,
   bool modelPar_given_FOL2,
   AdmsSensFadType & modelPar_VNSUBO,
   bool modelPar_given_VNSUBO,
   AdmsSensFadType & modelPar_NSLPO,
   bool modelPar_given_NSLPO,
   AdmsSensFadType & modelPar_DNSUBO,
   bool modelPar_given_DNSUBO,
   AdmsSensFadType & modelPar_DPHIBO,
   bool modelPar_given_DPHIBO,
   AdmsSensFadType & modelPar_DPHIBL,
   bool modelPar_given_DPHIBL,
   AdmsSensFadType & modelPar_DPHIBLEXP,
   bool modelPar_given_DPHIBLEXP,
   AdmsSensFadType & modelPar_DPHIBW,
   bool modelPar_given_DPHIBW,
   AdmsSensFadType & modelPar_DPHIBLW,
   bool modelPar_given_DPHIBLW,
   AdmsSensFadType & modelPar_NPO,
   bool modelPar_given_NPO,
   AdmsSensFadType & modelPar_NPL,
   bool modelPar_given_NPL,
   AdmsSensFadType & modelPar_CTO,
   bool modelPar_given_CTO,
   AdmsSensFadType & modelPar_CTL,
   bool modelPar_given_CTL,
   AdmsSensFadType & modelPar_CTLEXP,
   bool modelPar_given_CTLEXP,
   AdmsSensFadType & modelPar_CTW,
   bool modelPar_given_CTW,
   AdmsSensFadType & modelPar_CTLW,
   bool modelPar_given_CTLW,
   AdmsSensFadType & modelPar_TOXOVO,
   bool modelPar_given_TOXOVO,
   AdmsSensFadType & modelPar_TOXOVDO,
   bool modelPar_given_TOXOVDO,
   AdmsSensFadType & modelPar_LOV,
   bool modelPar_given_LOV,
   AdmsSensFadType & modelPar_LOVD,
   bool modelPar_given_LOVD,
   AdmsSensFadType & modelPar_NOVO,
   bool modelPar_given_NOVO,
   AdmsSensFadType & modelPar_NOVDO,
   bool modelPar_given_NOVDO,
   AdmsSensFadType & modelPar_CFL,
   bool modelPar_given_CFL,
   AdmsSensFadType & modelPar_CFLEXP,
   bool modelPar_given_CFLEXP,
   AdmsSensFadType & modelPar_CFW,
   bool modelPar_given_CFW,
   AdmsSensFadType & modelPar_CFBO,
   bool modelPar_given_CFBO,
   AdmsSensFadType & modelPar_UO,
   bool modelPar_given_UO,
   AdmsSensFadType & modelPar_FBET1,
   bool modelPar_given_FBET1,
   AdmsSensFadType & modelPar_FBET1W,
   bool modelPar_given_FBET1W,
   AdmsSensFadType & modelPar_LP1,
   bool modelPar_given_LP1,
   AdmsSensFadType & modelPar_LP1W,
   bool modelPar_given_LP1W,
   AdmsSensFadType & modelPar_FBET2,
   bool modelPar_given_FBET2,
   AdmsSensFadType & modelPar_LP2,
   bool modelPar_given_LP2,
   AdmsSensFadType & modelPar_BETW1,
   bool modelPar_given_BETW1,
   AdmsSensFadType & modelPar_BETW2,
   bool modelPar_given_BETW2,
   AdmsSensFadType & modelPar_WBET,
   bool modelPar_given_WBET,
   AdmsSensFadType & modelPar_STBETO,
   bool modelPar_given_STBETO,
   AdmsSensFadType & modelPar_STBETL,
   bool modelPar_given_STBETL,
   AdmsSensFadType & modelPar_STBETW,
   bool modelPar_given_STBETW,
   AdmsSensFadType & modelPar_STBETLW,
   bool modelPar_given_STBETLW,
   AdmsSensFadType & modelPar_MUEO,
   bool modelPar_given_MUEO,
   AdmsSensFadType & modelPar_MUEW,
   bool modelPar_given_MUEW,
   AdmsSensFadType & modelPar_STMUEO,
   bool modelPar_given_STMUEO,
   AdmsSensFadType & modelPar_THEMUO,
   bool modelPar_given_THEMUO,
   AdmsSensFadType & modelPar_STTHEMUO,
   bool modelPar_given_STTHEMUO,
   AdmsSensFadType & modelPar_CSO,
   bool modelPar_given_CSO,
   AdmsSensFadType & modelPar_CSL,
   bool modelPar_given_CSL,
   AdmsSensFadType & modelPar_CSLEXP,
   bool modelPar_given_CSLEXP,
   AdmsSensFadType & modelPar_CSW,
   bool modelPar_given_CSW,
   AdmsSensFadType & modelPar_CSLW,
   bool modelPar_given_CSLW,
   AdmsSensFadType & modelPar_STCSO,
   bool modelPar_given_STCSO,
   AdmsSensFadType & modelPar_XCORO,
   bool modelPar_given_XCORO,
   AdmsSensFadType & modelPar_XCORL,
   bool modelPar_given_XCORL,
   AdmsSensFadType & modelPar_XCORW,
   bool modelPar_given_XCORW,
   AdmsSensFadType & modelPar_XCORLW,
   bool modelPar_given_XCORLW,
   AdmsSensFadType & modelPar_STXCORO,
   bool modelPar_given_STXCORO,
   AdmsSensFadType & modelPar_FETAO,
   bool modelPar_given_FETAO,
   AdmsSensFadType & modelPar_RSW1,
   bool modelPar_given_RSW1,
   AdmsSensFadType & modelPar_RSW2,
   bool modelPar_given_RSW2,
   AdmsSensFadType & modelPar_STRSO,
   bool modelPar_given_STRSO,
   AdmsSensFadType & modelPar_RSBO,
   bool modelPar_given_RSBO,
   AdmsSensFadType & modelPar_RSGO,
   bool modelPar_given_RSGO,
   AdmsSensFadType & modelPar_THESATO,
   bool modelPar_given_THESATO,
   AdmsSensFadType & modelPar_THESATL,
   bool modelPar_given_THESATL,
   AdmsSensFadType & modelPar_THESATLEXP,
   bool modelPar_given_THESATLEXP,
   AdmsSensFadType & modelPar_THESATW,
   bool modelPar_given_THESATW,
   AdmsSensFadType & modelPar_THESATLW,
   bool modelPar_given_THESATLW,
   AdmsSensFadType & modelPar_STTHESATO,
   bool modelPar_given_STTHESATO,
   AdmsSensFadType & modelPar_STTHESATL,
   bool modelPar_given_STTHESATL,
   AdmsSensFadType & modelPar_STTHESATW,
   bool modelPar_given_STTHESATW,
   AdmsSensFadType & modelPar_STTHESATLW,
   bool modelPar_given_STTHESATLW,
   AdmsSensFadType & modelPar_THESATBO,
   bool modelPar_given_THESATBO,
   AdmsSensFadType & modelPar_THESATGO,
   bool modelPar_given_THESATGO,
   AdmsSensFadType & modelPar_AXO,
   bool modelPar_given_AXO,
   AdmsSensFadType & modelPar_AXL,
   bool modelPar_given_AXL,
   AdmsSensFadType & modelPar_ALPL,
   bool modelPar_given_ALPL,
   AdmsSensFadType & modelPar_ALPLEXP,
   bool modelPar_given_ALPLEXP,
   AdmsSensFadType & modelPar_ALPW,
   bool modelPar_given_ALPW,
   AdmsSensFadType & modelPar_ALP1L1,
   bool modelPar_given_ALP1L1,
   AdmsSensFadType & modelPar_ALP1LEXP,
   bool modelPar_given_ALP1LEXP,
   AdmsSensFadType & modelPar_ALP1L2,
   bool modelPar_given_ALP1L2,
   AdmsSensFadType & modelPar_ALP1W,
   bool modelPar_given_ALP1W,
   AdmsSensFadType & modelPar_ALP2L1,
   bool modelPar_given_ALP2L1,
   AdmsSensFadType & modelPar_ALP2LEXP,
   bool modelPar_given_ALP2LEXP,
   AdmsSensFadType & modelPar_ALP2L2,
   bool modelPar_given_ALP2L2,
   AdmsSensFadType & modelPar_ALP2W,
   bool modelPar_given_ALP2W,
   AdmsSensFadType & modelPar_VPO,
   bool modelPar_given_VPO,
   AdmsSensFadType & modelPar_A1O,
   bool modelPar_given_A1O,
   AdmsSensFadType & modelPar_A1L,
   bool modelPar_given_A1L,
   AdmsSensFadType & modelPar_A1W,
   bool modelPar_given_A1W,
   AdmsSensFadType & modelPar_A2O,
   bool modelPar_given_A2O,
   AdmsSensFadType & modelPar_STA2O,
   bool modelPar_given_STA2O,
   AdmsSensFadType & modelPar_A3O,
   bool modelPar_given_A3O,
   AdmsSensFadType & modelPar_A3L,
   bool modelPar_given_A3L,
   AdmsSensFadType & modelPar_A3W,
   bool modelPar_given_A3W,
   AdmsSensFadType & modelPar_A4O,
   bool modelPar_given_A4O,
   AdmsSensFadType & modelPar_A4L,
   bool modelPar_given_A4L,
   AdmsSensFadType & modelPar_A4W,
   bool modelPar_given_A4W,
   AdmsSensFadType & modelPar_GCOO,
   bool modelPar_given_GCOO,
   AdmsSensFadType & modelPar_IGINVLW,
   bool modelPar_given_IGINVLW,
   AdmsSensFadType & modelPar_IGOVW,
   bool modelPar_given_IGOVW,
   AdmsSensFadType & modelPar_IGOVDW,
   bool modelPar_given_IGOVDW,
   AdmsSensFadType & modelPar_STIGO,
   bool modelPar_given_STIGO,
   AdmsSensFadType & modelPar_GC2O,
   bool modelPar_given_GC2O,
   AdmsSensFadType & modelPar_GC3O,
   bool modelPar_given_GC3O,
   AdmsSensFadType & modelPar_CHIBO,
   bool modelPar_given_CHIBO,
   AdmsSensFadType & modelPar_AGIDLW,
   bool modelPar_given_AGIDLW,
   AdmsSensFadType & modelPar_AGIDLDW,
   bool modelPar_given_AGIDLDW,
   AdmsSensFadType & modelPar_BGIDLO,
   bool modelPar_given_BGIDLO,
   AdmsSensFadType & modelPar_BGIDLDO,
   bool modelPar_given_BGIDLDO,
   AdmsSensFadType & modelPar_STBGIDLO,
   bool modelPar_given_STBGIDLO,
   AdmsSensFadType & modelPar_STBGIDLDO,
   bool modelPar_given_STBGIDLDO,
   AdmsSensFadType & modelPar_CGIDLO,
   bool modelPar_given_CGIDLO,
   AdmsSensFadType & modelPar_CGIDLDO,
   bool modelPar_given_CGIDLDO,
   AdmsSensFadType & modelPar_CGBOVL,
   bool modelPar_given_CGBOVL,
   AdmsSensFadType & modelPar_CFRW,
   bool modelPar_given_CFRW,
   AdmsSensFadType & modelPar_CFRDW,
   bool modelPar_given_CFRDW,
   AdmsSensFadType & modelPar_FNTO,
   bool modelPar_given_FNTO,
   AdmsSensFadType & modelPar_FNTEXCL,
   bool modelPar_given_FNTEXCL,
   AdmsSensFadType & modelPar_NFALW,
   bool modelPar_given_NFALW,
   AdmsSensFadType & modelPar_NFBLW,
   bool modelPar_given_NFBLW,
   AdmsSensFadType & modelPar_NFCLW,
   bool modelPar_given_NFCLW,
   AdmsSensFadType & modelPar_EFO,
   bool modelPar_given_EFO,
   AdmsSensFadType & modelPar_LINTNOI,
   bool modelPar_given_LINTNOI,
   AdmsSensFadType & modelPar_ALPNOI,
   bool modelPar_given_ALPNOI,
   AdmsSensFadType & modelPar_DTA,
   bool modelPar_given_DTA,
   AdmsSensFadType & modelPar_KVTHOWEO,
   bool modelPar_given_KVTHOWEO,
   AdmsSensFadType & modelPar_KVTHOWEL,
   bool modelPar_given_KVTHOWEL,
   AdmsSensFadType & modelPar_KVTHOWEW,
   bool modelPar_given_KVTHOWEW,
   AdmsSensFadType & modelPar_KVTHOWELW,
   bool modelPar_given_KVTHOWELW,
   AdmsSensFadType & modelPar_KUOWEO,
   bool modelPar_given_KUOWEO,
   AdmsSensFadType & modelPar_KUOWEL,
   bool modelPar_given_KUOWEL,
   AdmsSensFadType & modelPar_KUOWEW,
   bool modelPar_given_KUOWEW,
   AdmsSensFadType & modelPar_KUOWELW,
   bool modelPar_given_KUOWELW,
   AdmsSensFadType & modelPar_LMIN,
   bool modelPar_given_LMIN,
   AdmsSensFadType & modelPar_LMAX,
   bool modelPar_given_LMAX,
   AdmsSensFadType & modelPar_WMIN,
   bool modelPar_given_WMIN,
   AdmsSensFadType & modelPar_WMAX,
   bool modelPar_given_WMAX,
   AdmsSensFadType & modelPar_RGO,
   bool modelPar_given_RGO,
   AdmsSensFadType & modelPar_RBULKO,
   bool modelPar_given_RBULKO,
   AdmsSensFadType & modelPar_RWELLO,
   bool modelPar_given_RWELLO,
   AdmsSensFadType & modelPar_RJUNSO,
   bool modelPar_given_RJUNSO,
   AdmsSensFadType & modelPar_RJUNDO,
   bool modelPar_given_RJUNDO,
   AdmsSensFadType & modelPar_RINT,
   bool modelPar_given_RINT,
   AdmsSensFadType & modelPar_RVPOLY,
   bool modelPar_given_RVPOLY,
   AdmsSensFadType & modelPar_RSHG,
   bool modelPar_given_RSHG,
   AdmsSensFadType & modelPar_DLSIL,
   bool modelPar_given_DLSIL,
   AdmsSensFadType & modelPar_SAREF,
   bool modelPar_given_SAREF,
   AdmsSensFadType & modelPar_SBREF,
   bool modelPar_given_SBREF,
   AdmsSensFadType & modelPar_WLOD,
   bool modelPar_given_WLOD,
   AdmsSensFadType & modelPar_KUO,
   bool modelPar_given_KUO,
   AdmsSensFadType & modelPar_KVSAT,
   bool modelPar_given_KVSAT,
   AdmsSensFadType & modelPar_TKUO,
   bool modelPar_given_TKUO,
   AdmsSensFadType & modelPar_LKUO,
   bool modelPar_given_LKUO,
   AdmsSensFadType & modelPar_WKUO,
   bool modelPar_given_WKUO,
   AdmsSensFadType & modelPar_PKUO,
   bool modelPar_given_PKUO,
   AdmsSensFadType & modelPar_LLODKUO,
   bool modelPar_given_LLODKUO,
   AdmsSensFadType & modelPar_WLODKUO,
   bool modelPar_given_WLODKUO,
   AdmsSensFadType & modelPar_KVTHO,
   bool modelPar_given_KVTHO,
   AdmsSensFadType & modelPar_LKVTHO,
   bool modelPar_given_LKVTHO,
   AdmsSensFadType & modelPar_WKVTHO,
   bool modelPar_given_WKVTHO,
   AdmsSensFadType & modelPar_PKVTHO,
   bool modelPar_given_PKVTHO,
   AdmsSensFadType & modelPar_LLODVTH,
   bool modelPar_given_LLODVTH,
   AdmsSensFadType & modelPar_WLODVTH,
   bool modelPar_given_WLODVTH,
   AdmsSensFadType & modelPar_STETAO,
   bool modelPar_given_STETAO,
   AdmsSensFadType & modelPar_LODETAO,
   bool modelPar_given_LODETAO,
   AdmsSensFadType & modelPar_SCREF,
   bool modelPar_given_SCREF,
   AdmsSensFadType & modelPar_WEB,
   bool modelPar_given_WEB,
   AdmsSensFadType & modelPar_WEC,
   bool modelPar_given_WEC,
   AdmsSensFadType & modelPar_IMAX,
   bool modelPar_given_IMAX,
   AdmsSensFadType & modelPar_TRJ,
   bool modelPar_given_TRJ,
   AdmsSensFadType & modelPar_CJORBOT,
   bool modelPar_given_CJORBOT,
   AdmsSensFadType & modelPar_CJORSTI,
   bool modelPar_given_CJORSTI,
   AdmsSensFadType & modelPar_CJORGAT,
   bool modelPar_given_CJORGAT,
   AdmsSensFadType & modelPar_VBIRBOT,
   bool modelPar_given_VBIRBOT,
   AdmsSensFadType & modelPar_VBIRSTI,
   bool modelPar_given_VBIRSTI,
   AdmsSensFadType & modelPar_VBIRGAT,
   bool modelPar_given_VBIRGAT,
   AdmsSensFadType & modelPar_PBOT,
   bool modelPar_given_PBOT,
   AdmsSensFadType & modelPar_PSTI,
   bool modelPar_given_PSTI,
   AdmsSensFadType & modelPar_PGAT,
   bool modelPar_given_PGAT,
   AdmsSensFadType & modelPar_PHIGBOT,
   bool modelPar_given_PHIGBOT,
   AdmsSensFadType & modelPar_PHIGSTI,
   bool modelPar_given_PHIGSTI,
   AdmsSensFadType & modelPar_PHIGGAT,
   bool modelPar_given_PHIGGAT,
   AdmsSensFadType & modelPar_IDSATRBOT,
   bool modelPar_given_IDSATRBOT,
   AdmsSensFadType & modelPar_IDSATRSTI,
   bool modelPar_given_IDSATRSTI,
   AdmsSensFadType & modelPar_IDSATRGAT,
   bool modelPar_given_IDSATRGAT,
   AdmsSensFadType & modelPar_CSRHBOT,
   bool modelPar_given_CSRHBOT,
   AdmsSensFadType & modelPar_CSRHSTI,
   bool modelPar_given_CSRHSTI,
   AdmsSensFadType & modelPar_CSRHGAT,
   bool modelPar_given_CSRHGAT,
   AdmsSensFadType & modelPar_XJUNSTI,
   bool modelPar_given_XJUNSTI,
   AdmsSensFadType & modelPar_XJUNGAT,
   bool modelPar_given_XJUNGAT,
   AdmsSensFadType & modelPar_CTATBOT,
   bool modelPar_given_CTATBOT,
   AdmsSensFadType & modelPar_CTATSTI,
   bool modelPar_given_CTATSTI,
   AdmsSensFadType & modelPar_CTATGAT,
   bool modelPar_given_CTATGAT,
   AdmsSensFadType & modelPar_MEFFTATBOT,
   bool modelPar_given_MEFFTATBOT,
   AdmsSensFadType & modelPar_MEFFTATSTI,
   bool modelPar_given_MEFFTATSTI,
   AdmsSensFadType & modelPar_MEFFTATGAT,
   bool modelPar_given_MEFFTATGAT,
   AdmsSensFadType & modelPar_CBBTBOT,
   bool modelPar_given_CBBTBOT,
   AdmsSensFadType & modelPar_CBBTSTI,
   bool modelPar_given_CBBTSTI,
   AdmsSensFadType & modelPar_CBBTGAT,
   bool modelPar_given_CBBTGAT,
   AdmsSensFadType & modelPar_FBBTRBOT,
   bool modelPar_given_FBBTRBOT,
   AdmsSensFadType & modelPar_FBBTRSTI,
   bool modelPar_given_FBBTRSTI,
   AdmsSensFadType & modelPar_FBBTRGAT,
   bool modelPar_given_FBBTRGAT,
   AdmsSensFadType & modelPar_STFBBTBOT,
   bool modelPar_given_STFBBTBOT,
   AdmsSensFadType & modelPar_STFBBTSTI,
   bool modelPar_given_STFBBTSTI,
   AdmsSensFadType & modelPar_STFBBTGAT,
   bool modelPar_given_STFBBTGAT,
   AdmsSensFadType & modelPar_VBRBOT,
   bool modelPar_given_VBRBOT,
   AdmsSensFadType & modelPar_VBRSTI,
   bool modelPar_given_VBRSTI,
   AdmsSensFadType & modelPar_VBRGAT,
   bool modelPar_given_VBRGAT,
   AdmsSensFadType & modelPar_PBRBOT,
   bool modelPar_given_PBRBOT,
   AdmsSensFadType & modelPar_PBRSTI,
   bool modelPar_given_PBRSTI,
   AdmsSensFadType & modelPar_PBRGAT,
   bool modelPar_given_PBRGAT,
   AdmsSensFadType & modelPar_CJORBOTD,
   bool modelPar_given_CJORBOTD,
   AdmsSensFadType & modelPar_CJORSTID,
   bool modelPar_given_CJORSTID,
   AdmsSensFadType & modelPar_CJORGATD,
   bool modelPar_given_CJORGATD,
   AdmsSensFadType & modelPar_VBIRBOTD,
   bool modelPar_given_VBIRBOTD,
   AdmsSensFadType & modelPar_VBIRSTID,
   bool modelPar_given_VBIRSTID,
   AdmsSensFadType & modelPar_VBIRGATD,
   bool modelPar_given_VBIRGATD,
   AdmsSensFadType & modelPar_PBOTD,
   bool modelPar_given_PBOTD,
   AdmsSensFadType & modelPar_PSTID,
   bool modelPar_given_PSTID,
   AdmsSensFadType & modelPar_PGATD,
   bool modelPar_given_PGATD,
   AdmsSensFadType & modelPar_PHIGBOTD,
   bool modelPar_given_PHIGBOTD,
   AdmsSensFadType & modelPar_PHIGSTID,
   bool modelPar_given_PHIGSTID,
   AdmsSensFadType & modelPar_PHIGGATD,
   bool modelPar_given_PHIGGATD,
   AdmsSensFadType & modelPar_IDSATRBOTD,
   bool modelPar_given_IDSATRBOTD,
   AdmsSensFadType & modelPar_IDSATRSTID,
   bool modelPar_given_IDSATRSTID,
   AdmsSensFadType & modelPar_IDSATRGATD,
   bool modelPar_given_IDSATRGATD,
   AdmsSensFadType & modelPar_CSRHBOTD,
   bool modelPar_given_CSRHBOTD,
   AdmsSensFadType & modelPar_CSRHSTID,
   bool modelPar_given_CSRHSTID,
   AdmsSensFadType & modelPar_CSRHGATD,
   bool modelPar_given_CSRHGATD,
   AdmsSensFadType & modelPar_XJUNSTID,
   bool modelPar_given_XJUNSTID,
   AdmsSensFadType & modelPar_XJUNGATD,
   bool modelPar_given_XJUNGATD,
   AdmsSensFadType & modelPar_CTATBOTD,
   bool modelPar_given_CTATBOTD,
   AdmsSensFadType & modelPar_CTATSTID,
   bool modelPar_given_CTATSTID,
   AdmsSensFadType & modelPar_CTATGATD,
   bool modelPar_given_CTATGATD,
   AdmsSensFadType & modelPar_MEFFTATBOTD,
   bool modelPar_given_MEFFTATBOTD,
   AdmsSensFadType & modelPar_MEFFTATSTID,
   bool modelPar_given_MEFFTATSTID,
   AdmsSensFadType & modelPar_MEFFTATGATD,
   bool modelPar_given_MEFFTATGATD,
   AdmsSensFadType & modelPar_CBBTBOTD,
   bool modelPar_given_CBBTBOTD,
   AdmsSensFadType & modelPar_CBBTSTID,
   bool modelPar_given_CBBTSTID,
   AdmsSensFadType & modelPar_CBBTGATD,
   bool modelPar_given_CBBTGATD,
   AdmsSensFadType & modelPar_FBBTRBOTD,
   bool modelPar_given_FBBTRBOTD,
   AdmsSensFadType & modelPar_FBBTRSTID,
   bool modelPar_given_FBBTRSTID,
   AdmsSensFadType & modelPar_FBBTRGATD,
   bool modelPar_given_FBBTRGATD,
   AdmsSensFadType & modelPar_STFBBTBOTD,
   bool modelPar_given_STFBBTBOTD,
   AdmsSensFadType & modelPar_STFBBTSTID,
   bool modelPar_given_STFBBTSTID,
   AdmsSensFadType & modelPar_STFBBTGATD,
   bool modelPar_given_STFBBTGATD,
   AdmsSensFadType & modelPar_VBRBOTD,
   bool modelPar_given_VBRBOTD,
   AdmsSensFadType & modelPar_VBRSTID,
   bool modelPar_given_VBRSTID,
   AdmsSensFadType & modelPar_VBRGATD,
   bool modelPar_given_VBRGATD,
   AdmsSensFadType & modelPar_PBRBOTD,
   bool modelPar_given_PBRBOTD,
   AdmsSensFadType & modelPar_PBRSTID,
   bool modelPar_given_PBRSTID,
   AdmsSensFadType & modelPar_PBRGATD,
   bool modelPar_given_PBRGATD,
   AdmsSensFadType & modelPar_SWJUNEXP,
   bool modelPar_given_SWJUNEXP,
   AdmsSensFadType & modelPar_VJUNREF,
   bool modelPar_given_VJUNREF,
   AdmsSensFadType & modelPar_FJUNQ,
   bool modelPar_given_FJUNQ,
   AdmsSensFadType & modelPar_VJUNREFD,
   bool modelPar_given_VJUNREFD,
   AdmsSensFadType & modelPar_FJUNQD,
   bool modelPar_given_FJUNQD// model variables
   ,
   // reals
   AdmsSensFadType & modelVar_NSUBO_i,
   AdmsSensFadType & modelVar_WSEG_i,
   AdmsSensFadType & modelVar_NPCK_i,
   AdmsSensFadType & modelVar_WSEGP_i,
   AdmsSensFadType & modelVar_LPCK_i,
   AdmsSensFadType & modelVar_LOV_i,
   AdmsSensFadType & modelVar_LOVD_i,
   AdmsSensFadType & modelVar_LP1_i,
   AdmsSensFadType & modelVar_LP2_i,
   AdmsSensFadType & modelVar_WBET_i,
   AdmsSensFadType & modelVar_AXL_i,
   AdmsSensFadType & modelVar_ALP1L2_i,
   AdmsSensFadType & modelVar_ALP2L2_i,
   AdmsSensFadType & modelVar_RINT_i,
   AdmsSensFadType & modelVar_RVPOLY_i,
   AdmsSensFadType & modelVar_SAREF_i,
   AdmsSensFadType & modelVar_SBREF_i,
   AdmsSensFadType & modelVar_KVSAT_i,
   AdmsSensFadType & modelVar_LLODKUO_i,
   AdmsSensFadType & modelVar_WLODKUO_i,
   AdmsSensFadType & modelVar_LLODVTH_i,
   AdmsSensFadType & modelVar_WLODVTH_i,
   AdmsSensFadType & modelVar_LODETAO_i,
   AdmsSensFadType & modelVar_SCREF_i,
   AdmsSensFadType & modelVar_WEB_i,
   AdmsSensFadType & modelVar_WEC_i,
   AdmsSensFadType & modelVar_TOX_i,
   AdmsSensFadType & modelVar_EPSROX_i,
   AdmsSensFadType & modelVar_TOXOV_i,
   AdmsSensFadType & modelVar_TOXOVD_i,
   AdmsSensFadType & modelVar_TKD,
   AdmsSensFadType & modelVar_dT,
   AdmsSensFadType & modelVar_rT,
   AdmsSensFadType & modelVar_rTn,
   AdmsSensFadType & modelVar_EPSSI,
   AdmsSensFadType & modelVar_phit,
   AdmsSensFadType & modelVar_inv_phit,
   AdmsSensFadType & modelVar_Eg,
   AdmsSensFadType & modelVar_phibFac,
   AdmsSensFadType & modelVar_nt0,
   AdmsSensFadType & modelVar_IMAX_i,
   AdmsSensFadType & modelVar_VBIRBOT_i,
   AdmsSensFadType & modelVar_VBIRSTI_i,
   AdmsSensFadType & modelVar_VBIRGAT_i,
   AdmsSensFadType & modelVar_PBOT_i,
   AdmsSensFadType & modelVar_PSTI_i,
   AdmsSensFadType & modelVar_PGAT_i,
   AdmsSensFadType & modelVar_CSRHBOT_i,
   AdmsSensFadType & modelVar_CSRHSTI_i,
   AdmsSensFadType & modelVar_CSRHGAT_i,
   AdmsSensFadType & modelVar_CTATBOT_i,
   AdmsSensFadType & modelVar_CTATSTI_i,
   AdmsSensFadType & modelVar_CTATGAT_i,
   AdmsSensFadType & modelVar_CBBTBOT_i,
   AdmsSensFadType & modelVar_CBBTSTI_i,
   AdmsSensFadType & modelVar_CBBTGAT_i,
   AdmsSensFadType & modelVar_VBRBOT_i,
   AdmsSensFadType & modelVar_VBRSTI_i,
   AdmsSensFadType & modelVar_VBRGAT_i,
   AdmsSensFadType & modelVar_PBRBOT_i,
   AdmsSensFadType & modelVar_PBRSTI_i,
   AdmsSensFadType & modelVar_PBRGAT_i,
   AdmsSensFadType & modelVar_SWJUNEXP_i,
   AdmsSensFadType & modelVar_VJUNREF_i,
   AdmsSensFadType & modelVar_FJUNQ_i,
   AdmsSensFadType & modelVar_phitr,
   AdmsSensFadType & modelVar_phitd,
   AdmsSensFadType & modelVar_phitdinv,
   AdmsSensFadType & modelVar_perfc,
   AdmsSensFadType & modelVar_berfc,
   AdmsSensFadType & modelVar_cerfc,
   AdmsSensFadType & modelVar_ftdbot,
   AdmsSensFadType & modelVar_ftdsti,
   AdmsSensFadType & modelVar_ftdgat,
   AdmsSensFadType & modelVar_idsatbot,
   AdmsSensFadType & modelVar_idsatsti,
   AdmsSensFadType & modelVar_idsatgat,
   AdmsSensFadType & modelVar_vbibot,
   AdmsSensFadType & modelVar_vbisti,
   AdmsSensFadType & modelVar_vbigat,
   AdmsSensFadType & modelVar_vbiinvbot,
   AdmsSensFadType & modelVar_vbiinvsti,
   AdmsSensFadType & modelVar_vbiinvgat,
   AdmsSensFadType & modelVar_one_minus_PBOT,
   AdmsSensFadType & modelVar_one_minus_PSTI,
   AdmsSensFadType & modelVar_one_minus_PGAT,
   AdmsSensFadType & modelVar_one_over_one_minus_PBOT,
   AdmsSensFadType & modelVar_one_over_one_minus_PSTI,
   AdmsSensFadType & modelVar_one_over_one_minus_PGAT,
   AdmsSensFadType & modelVar_cjobot,
   AdmsSensFadType & modelVar_cjosti,
   AdmsSensFadType & modelVar_cjogat,
   AdmsSensFadType & modelVar_qprefbot,
   AdmsSensFadType & modelVar_qprefsti,
   AdmsSensFadType & modelVar_qprefgat,
   AdmsSensFadType & modelVar_qpref2bot,
   AdmsSensFadType & modelVar_qpref2sti,
   AdmsSensFadType & modelVar_qpref2gat,
   AdmsSensFadType & modelVar_wdepnulrbot,
   AdmsSensFadType & modelVar_wdepnulrsti,
   AdmsSensFadType & modelVar_wdepnulrgat,
   AdmsSensFadType & modelVar_wdepnulrinvbot,
   AdmsSensFadType & modelVar_wdepnulrinvsti,
   AdmsSensFadType & modelVar_wdepnulrinvgat,
   AdmsSensFadType & modelVar_VBIRBOTinv,
   AdmsSensFadType & modelVar_VBIRSTIinv,
   AdmsSensFadType & modelVar_VBIRGATinv,
   AdmsSensFadType & modelVar_atatbot,
   AdmsSensFadType & modelVar_atatsti,
   AdmsSensFadType & modelVar_atatgat,
   AdmsSensFadType & modelVar_btatpartbot,
   AdmsSensFadType & modelVar_btatpartsti,
   AdmsSensFadType & modelVar_btatpartgat,
   AdmsSensFadType & modelVar_fbbtbot,
   AdmsSensFadType & modelVar_fbbtsti,
   AdmsSensFadType & modelVar_fbbtgat,
   AdmsSensFadType & modelVar_fstopbot,
   AdmsSensFadType & modelVar_fstopsti,
   AdmsSensFadType & modelVar_fstopgat,
   AdmsSensFadType & modelVar_VBRinvbot,
   AdmsSensFadType & modelVar_VBRinvsti,
   AdmsSensFadType & modelVar_VBRinvgat,
   AdmsSensFadType & modelVar_slopebot,
   AdmsSensFadType & modelVar_slopesti,
   AdmsSensFadType & modelVar_slopegat,
   AdmsSensFadType & modelVar_VBIRBOTD_i,
   AdmsSensFadType & modelVar_VBIRSTID_i,
   AdmsSensFadType & modelVar_VBIRGATD_i,
   AdmsSensFadType & modelVar_PBOTD_i,
   AdmsSensFadType & modelVar_PSTID_i,
   AdmsSensFadType & modelVar_PGATD_i,
   AdmsSensFadType & modelVar_CSRHBOTD_i,
   AdmsSensFadType & modelVar_CSRHSTID_i,
   AdmsSensFadType & modelVar_CSRHGATD_i,
   AdmsSensFadType & modelVar_CTATBOTD_i,
   AdmsSensFadType & modelVar_CTATSTID_i,
   AdmsSensFadType & modelVar_CTATGATD_i,
   AdmsSensFadType & modelVar_CBBTBOTD_i,
   AdmsSensFadType & modelVar_CBBTSTID_i,
   AdmsSensFadType & modelVar_CBBTGATD_i,
   AdmsSensFadType & modelVar_VBRBOTD_i,
   AdmsSensFadType & modelVar_VBRSTID_i,
   AdmsSensFadType & modelVar_VBRGATD_i,
   AdmsSensFadType & modelVar_PBRBOTD_i,
   AdmsSensFadType & modelVar_PBRSTID_i,
   AdmsSensFadType & modelVar_PBRGATD_i,
   AdmsSensFadType & modelVar_VJUNREFD_i,
   AdmsSensFadType & modelVar_FJUNQD_i,
   AdmsSensFadType & modelVar_ftdbot_d,
   AdmsSensFadType & modelVar_ftdsti_d,
   AdmsSensFadType & modelVar_ftdgat_d,
   AdmsSensFadType & modelVar_idsatbot_d,
   AdmsSensFadType & modelVar_idsatsti_d,
   AdmsSensFadType & modelVar_idsatgat_d,
   AdmsSensFadType & modelVar_vbibot_d,
   AdmsSensFadType & modelVar_vbisti_d,
   AdmsSensFadType & modelVar_vbigat_d,
   AdmsSensFadType & modelVar_vbiinvbot_d,
   AdmsSensFadType & modelVar_vbiinvsti_d,
   AdmsSensFadType & modelVar_vbiinvgat_d,
   AdmsSensFadType & modelVar_one_minus_PBOT_d,
   AdmsSensFadType & modelVar_one_minus_PSTI_d,
   AdmsSensFadType & modelVar_one_minus_PGAT_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PBOT_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PSTI_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PGAT_d,
   AdmsSensFadType & modelVar_cjobot_d,
   AdmsSensFadType & modelVar_cjosti_d,
   AdmsSensFadType & modelVar_cjogat_d,
   AdmsSensFadType & modelVar_qprefbot_d,
   AdmsSensFadType & modelVar_qprefsti_d,
   AdmsSensFadType & modelVar_qprefgat_d,
   AdmsSensFadType & modelVar_qpref2bot_d,
   AdmsSensFadType & modelVar_qpref2sti_d,
   AdmsSensFadType & modelVar_qpref2gat_d,
   AdmsSensFadType & modelVar_wdepnulrbot_d,
   AdmsSensFadType & modelVar_wdepnulrsti_d,
   AdmsSensFadType & modelVar_wdepnulrgat_d,
   AdmsSensFadType & modelVar_wdepnulrinvbot_d,
   AdmsSensFadType & modelVar_wdepnulrinvsti_d,
   AdmsSensFadType & modelVar_wdepnulrinvgat_d,
   AdmsSensFadType & modelVar_VBIRBOTinv_d,
   AdmsSensFadType & modelVar_VBIRSTIinv_d,
   AdmsSensFadType & modelVar_VBIRGATinv_d,
   AdmsSensFadType & modelVar_atatbot_d,
   AdmsSensFadType & modelVar_atatsti_d,
   AdmsSensFadType & modelVar_atatgat_d,
   AdmsSensFadType & modelVar_btatpartbot_d,
   AdmsSensFadType & modelVar_btatpartsti_d,
   AdmsSensFadType & modelVar_btatpartgat_d,
   AdmsSensFadType & modelVar_fbbtbot_d,
   AdmsSensFadType & modelVar_fbbtsti_d,
   AdmsSensFadType & modelVar_fbbtgat_d,
   AdmsSensFadType & modelVar_fstopbot_d,
   AdmsSensFadType & modelVar_fstopsti_d,
   AdmsSensFadType & modelVar_fstopgat_d,
   AdmsSensFadType & modelVar_VBRinvbot_d,
   AdmsSensFadType & modelVar_VBRinvsti_d,
   AdmsSensFadType & modelVar_VBRinvgat_d,
   AdmsSensFadType & modelVar_slopebot_d,
   AdmsSensFadType & modelVar_slopesti_d,
   AdmsSensFadType & modelVar_slopegat_d,
   // non-reals
   int modelVar_CHNL_TYPE,
   double admsTemperature,double adms_vt_nom, double ADMSgmin_arg, const Instance & theInstance);

void evaluateInitialModel(
   // model parameters
   // reals
   AdmsSensFadType & modelPar_LEVEL,
   bool modelPar_given_LEVEL,
   AdmsSensFadType & modelPar_TYPE,
   bool modelPar_given_TYPE,
   AdmsSensFadType & modelPar_TR,
   bool modelPar_given_TR,
   AdmsSensFadType & modelPar_SWIGATE,
   bool modelPar_given_SWIGATE,
   AdmsSensFadType & modelPar_SWIMPACT,
   bool modelPar_given_SWIMPACT,
   AdmsSensFadType & modelPar_SWGIDL,
   bool modelPar_given_SWGIDL,
   AdmsSensFadType & modelPar_SWJUNCAP,
   bool modelPar_given_SWJUNCAP,
   AdmsSensFadType & modelPar_SWJUNASYM,
   bool modelPar_given_SWJUNASYM,
   AdmsSensFadType & modelPar_QMC,
   bool modelPar_given_QMC,
   AdmsSensFadType & modelPar_LVARO,
   bool modelPar_given_LVARO,
   AdmsSensFadType & modelPar_LVARL,
   bool modelPar_given_LVARL,
   AdmsSensFadType & modelPar_LVARW,
   bool modelPar_given_LVARW,
   AdmsSensFadType & modelPar_LAP,
   bool modelPar_given_LAP,
   AdmsSensFadType & modelPar_WVARO,
   bool modelPar_given_WVARO,
   AdmsSensFadType & modelPar_WVARL,
   bool modelPar_given_WVARL,
   AdmsSensFadType & modelPar_WVARW,
   bool modelPar_given_WVARW,
   AdmsSensFadType & modelPar_WOT,
   bool modelPar_given_WOT,
   AdmsSensFadType & modelPar_DLQ,
   bool modelPar_given_DLQ,
   AdmsSensFadType & modelPar_DWQ,
   bool modelPar_given_DWQ,
   AdmsSensFadType & modelPar_VFBO,
   bool modelPar_given_VFBO,
   AdmsSensFadType & modelPar_VFBL,
   bool modelPar_given_VFBL,
   AdmsSensFadType & modelPar_VFBW,
   bool modelPar_given_VFBW,
   AdmsSensFadType & modelPar_VFBLW,
   bool modelPar_given_VFBLW,
   AdmsSensFadType & modelPar_STVFBO,
   bool modelPar_given_STVFBO,
   AdmsSensFadType & modelPar_STVFBL,
   bool modelPar_given_STVFBL,
   AdmsSensFadType & modelPar_STVFBW,
   bool modelPar_given_STVFBW,
   AdmsSensFadType & modelPar_STVFBLW,
   bool modelPar_given_STVFBLW,
   AdmsSensFadType & modelPar_TOXO,
   bool modelPar_given_TOXO,
   AdmsSensFadType & modelPar_EPSROXO,
   bool modelPar_given_EPSROXO,
   AdmsSensFadType & modelPar_NSUBO,
   bool modelPar_given_NSUBO,
   AdmsSensFadType & modelPar_NSUBW,
   bool modelPar_given_NSUBW,
   AdmsSensFadType & modelPar_WSEG,
   bool modelPar_given_WSEG,
   AdmsSensFadType & modelPar_NPCK,
   bool modelPar_given_NPCK,
   AdmsSensFadType & modelPar_NPCKW,
   bool modelPar_given_NPCKW,
   AdmsSensFadType & modelPar_WSEGP,
   bool modelPar_given_WSEGP,
   AdmsSensFadType & modelPar_LPCK,
   bool modelPar_given_LPCK,
   AdmsSensFadType & modelPar_LPCKW,
   bool modelPar_given_LPCKW,
   AdmsSensFadType & modelPar_FOL1,
   bool modelPar_given_FOL1,
   AdmsSensFadType & modelPar_FOL2,
   bool modelPar_given_FOL2,
   AdmsSensFadType & modelPar_VNSUBO,
   bool modelPar_given_VNSUBO,
   AdmsSensFadType & modelPar_NSLPO,
   bool modelPar_given_NSLPO,
   AdmsSensFadType & modelPar_DNSUBO,
   bool modelPar_given_DNSUBO,
   AdmsSensFadType & modelPar_DPHIBO,
   bool modelPar_given_DPHIBO,
   AdmsSensFadType & modelPar_DPHIBL,
   bool modelPar_given_DPHIBL,
   AdmsSensFadType & modelPar_DPHIBLEXP,
   bool modelPar_given_DPHIBLEXP,
   AdmsSensFadType & modelPar_DPHIBW,
   bool modelPar_given_DPHIBW,
   AdmsSensFadType & modelPar_DPHIBLW,
   bool modelPar_given_DPHIBLW,
   AdmsSensFadType & modelPar_NPO,
   bool modelPar_given_NPO,
   AdmsSensFadType & modelPar_NPL,
   bool modelPar_given_NPL,
   AdmsSensFadType & modelPar_CTO,
   bool modelPar_given_CTO,
   AdmsSensFadType & modelPar_CTL,
   bool modelPar_given_CTL,
   AdmsSensFadType & modelPar_CTLEXP,
   bool modelPar_given_CTLEXP,
   AdmsSensFadType & modelPar_CTW,
   bool modelPar_given_CTW,
   AdmsSensFadType & modelPar_CTLW,
   bool modelPar_given_CTLW,
   AdmsSensFadType & modelPar_TOXOVO,
   bool modelPar_given_TOXOVO,
   AdmsSensFadType & modelPar_TOXOVDO,
   bool modelPar_given_TOXOVDO,
   AdmsSensFadType & modelPar_LOV,
   bool modelPar_given_LOV,
   AdmsSensFadType & modelPar_LOVD,
   bool modelPar_given_LOVD,
   AdmsSensFadType & modelPar_NOVO,
   bool modelPar_given_NOVO,
   AdmsSensFadType & modelPar_NOVDO,
   bool modelPar_given_NOVDO,
   AdmsSensFadType & modelPar_CFL,
   bool modelPar_given_CFL,
   AdmsSensFadType & modelPar_CFLEXP,
   bool modelPar_given_CFLEXP,
   AdmsSensFadType & modelPar_CFW,
   bool modelPar_given_CFW,
   AdmsSensFadType & modelPar_CFBO,
   bool modelPar_given_CFBO,
   AdmsSensFadType & modelPar_UO,
   bool modelPar_given_UO,
   AdmsSensFadType & modelPar_FBET1,
   bool modelPar_given_FBET1,
   AdmsSensFadType & modelPar_FBET1W,
   bool modelPar_given_FBET1W,
   AdmsSensFadType & modelPar_LP1,
   bool modelPar_given_LP1,
   AdmsSensFadType & modelPar_LP1W,
   bool modelPar_given_LP1W,
   AdmsSensFadType & modelPar_FBET2,
   bool modelPar_given_FBET2,
   AdmsSensFadType & modelPar_LP2,
   bool modelPar_given_LP2,
   AdmsSensFadType & modelPar_BETW1,
   bool modelPar_given_BETW1,
   AdmsSensFadType & modelPar_BETW2,
   bool modelPar_given_BETW2,
   AdmsSensFadType & modelPar_WBET,
   bool modelPar_given_WBET,
   AdmsSensFadType & modelPar_STBETO,
   bool modelPar_given_STBETO,
   AdmsSensFadType & modelPar_STBETL,
   bool modelPar_given_STBETL,
   AdmsSensFadType & modelPar_STBETW,
   bool modelPar_given_STBETW,
   AdmsSensFadType & modelPar_STBETLW,
   bool modelPar_given_STBETLW,
   AdmsSensFadType & modelPar_MUEO,
   bool modelPar_given_MUEO,
   AdmsSensFadType & modelPar_MUEW,
   bool modelPar_given_MUEW,
   AdmsSensFadType & modelPar_STMUEO,
   bool modelPar_given_STMUEO,
   AdmsSensFadType & modelPar_THEMUO,
   bool modelPar_given_THEMUO,
   AdmsSensFadType & modelPar_STTHEMUO,
   bool modelPar_given_STTHEMUO,
   AdmsSensFadType & modelPar_CSO,
   bool modelPar_given_CSO,
   AdmsSensFadType & modelPar_CSL,
   bool modelPar_given_CSL,
   AdmsSensFadType & modelPar_CSLEXP,
   bool modelPar_given_CSLEXP,
   AdmsSensFadType & modelPar_CSW,
   bool modelPar_given_CSW,
   AdmsSensFadType & modelPar_CSLW,
   bool modelPar_given_CSLW,
   AdmsSensFadType & modelPar_STCSO,
   bool modelPar_given_STCSO,
   AdmsSensFadType & modelPar_XCORO,
   bool modelPar_given_XCORO,
   AdmsSensFadType & modelPar_XCORL,
   bool modelPar_given_XCORL,
   AdmsSensFadType & modelPar_XCORW,
   bool modelPar_given_XCORW,
   AdmsSensFadType & modelPar_XCORLW,
   bool modelPar_given_XCORLW,
   AdmsSensFadType & modelPar_STXCORO,
   bool modelPar_given_STXCORO,
   AdmsSensFadType & modelPar_FETAO,
   bool modelPar_given_FETAO,
   AdmsSensFadType & modelPar_RSW1,
   bool modelPar_given_RSW1,
   AdmsSensFadType & modelPar_RSW2,
   bool modelPar_given_RSW2,
   AdmsSensFadType & modelPar_STRSO,
   bool modelPar_given_STRSO,
   AdmsSensFadType & modelPar_RSBO,
   bool modelPar_given_RSBO,
   AdmsSensFadType & modelPar_RSGO,
   bool modelPar_given_RSGO,
   AdmsSensFadType & modelPar_THESATO,
   bool modelPar_given_THESATO,
   AdmsSensFadType & modelPar_THESATL,
   bool modelPar_given_THESATL,
   AdmsSensFadType & modelPar_THESATLEXP,
   bool modelPar_given_THESATLEXP,
   AdmsSensFadType & modelPar_THESATW,
   bool modelPar_given_THESATW,
   AdmsSensFadType & modelPar_THESATLW,
   bool modelPar_given_THESATLW,
   AdmsSensFadType & modelPar_STTHESATO,
   bool modelPar_given_STTHESATO,
   AdmsSensFadType & modelPar_STTHESATL,
   bool modelPar_given_STTHESATL,
   AdmsSensFadType & modelPar_STTHESATW,
   bool modelPar_given_STTHESATW,
   AdmsSensFadType & modelPar_STTHESATLW,
   bool modelPar_given_STTHESATLW,
   AdmsSensFadType & modelPar_THESATBO,
   bool modelPar_given_THESATBO,
   AdmsSensFadType & modelPar_THESATGO,
   bool modelPar_given_THESATGO,
   AdmsSensFadType & modelPar_AXO,
   bool modelPar_given_AXO,
   AdmsSensFadType & modelPar_AXL,
   bool modelPar_given_AXL,
   AdmsSensFadType & modelPar_ALPL,
   bool modelPar_given_ALPL,
   AdmsSensFadType & modelPar_ALPLEXP,
   bool modelPar_given_ALPLEXP,
   AdmsSensFadType & modelPar_ALPW,
   bool modelPar_given_ALPW,
   AdmsSensFadType & modelPar_ALP1L1,
   bool modelPar_given_ALP1L1,
   AdmsSensFadType & modelPar_ALP1LEXP,
   bool modelPar_given_ALP1LEXP,
   AdmsSensFadType & modelPar_ALP1L2,
   bool modelPar_given_ALP1L2,
   AdmsSensFadType & modelPar_ALP1W,
   bool modelPar_given_ALP1W,
   AdmsSensFadType & modelPar_ALP2L1,
   bool modelPar_given_ALP2L1,
   AdmsSensFadType & modelPar_ALP2LEXP,
   bool modelPar_given_ALP2LEXP,
   AdmsSensFadType & modelPar_ALP2L2,
   bool modelPar_given_ALP2L2,
   AdmsSensFadType & modelPar_ALP2W,
   bool modelPar_given_ALP2W,
   AdmsSensFadType & modelPar_VPO,
   bool modelPar_given_VPO,
   AdmsSensFadType & modelPar_A1O,
   bool modelPar_given_A1O,
   AdmsSensFadType & modelPar_A1L,
   bool modelPar_given_A1L,
   AdmsSensFadType & modelPar_A1W,
   bool modelPar_given_A1W,
   AdmsSensFadType & modelPar_A2O,
   bool modelPar_given_A2O,
   AdmsSensFadType & modelPar_STA2O,
   bool modelPar_given_STA2O,
   AdmsSensFadType & modelPar_A3O,
   bool modelPar_given_A3O,
   AdmsSensFadType & modelPar_A3L,
   bool modelPar_given_A3L,
   AdmsSensFadType & modelPar_A3W,
   bool modelPar_given_A3W,
   AdmsSensFadType & modelPar_A4O,
   bool modelPar_given_A4O,
   AdmsSensFadType & modelPar_A4L,
   bool modelPar_given_A4L,
   AdmsSensFadType & modelPar_A4W,
   bool modelPar_given_A4W,
   AdmsSensFadType & modelPar_GCOO,
   bool modelPar_given_GCOO,
   AdmsSensFadType & modelPar_IGINVLW,
   bool modelPar_given_IGINVLW,
   AdmsSensFadType & modelPar_IGOVW,
   bool modelPar_given_IGOVW,
   AdmsSensFadType & modelPar_IGOVDW,
   bool modelPar_given_IGOVDW,
   AdmsSensFadType & modelPar_STIGO,
   bool modelPar_given_STIGO,
   AdmsSensFadType & modelPar_GC2O,
   bool modelPar_given_GC2O,
   AdmsSensFadType & modelPar_GC3O,
   bool modelPar_given_GC3O,
   AdmsSensFadType & modelPar_CHIBO,
   bool modelPar_given_CHIBO,
   AdmsSensFadType & modelPar_AGIDLW,
   bool modelPar_given_AGIDLW,
   AdmsSensFadType & modelPar_AGIDLDW,
   bool modelPar_given_AGIDLDW,
   AdmsSensFadType & modelPar_BGIDLO,
   bool modelPar_given_BGIDLO,
   AdmsSensFadType & modelPar_BGIDLDO,
   bool modelPar_given_BGIDLDO,
   AdmsSensFadType & modelPar_STBGIDLO,
   bool modelPar_given_STBGIDLO,
   AdmsSensFadType & modelPar_STBGIDLDO,
   bool modelPar_given_STBGIDLDO,
   AdmsSensFadType & modelPar_CGIDLO,
   bool modelPar_given_CGIDLO,
   AdmsSensFadType & modelPar_CGIDLDO,
   bool modelPar_given_CGIDLDO,
   AdmsSensFadType & modelPar_CGBOVL,
   bool modelPar_given_CGBOVL,
   AdmsSensFadType & modelPar_CFRW,
   bool modelPar_given_CFRW,
   AdmsSensFadType & modelPar_CFRDW,
   bool modelPar_given_CFRDW,
   AdmsSensFadType & modelPar_FNTO,
   bool modelPar_given_FNTO,
   AdmsSensFadType & modelPar_FNTEXCL,
   bool modelPar_given_FNTEXCL,
   AdmsSensFadType & modelPar_NFALW,
   bool modelPar_given_NFALW,
   AdmsSensFadType & modelPar_NFBLW,
   bool modelPar_given_NFBLW,
   AdmsSensFadType & modelPar_NFCLW,
   bool modelPar_given_NFCLW,
   AdmsSensFadType & modelPar_EFO,
   bool modelPar_given_EFO,
   AdmsSensFadType & modelPar_LINTNOI,
   bool modelPar_given_LINTNOI,
   AdmsSensFadType & modelPar_ALPNOI,
   bool modelPar_given_ALPNOI,
   AdmsSensFadType & modelPar_DTA,
   bool modelPar_given_DTA,
   AdmsSensFadType & modelPar_KVTHOWEO,
   bool modelPar_given_KVTHOWEO,
   AdmsSensFadType & modelPar_KVTHOWEL,
   bool modelPar_given_KVTHOWEL,
   AdmsSensFadType & modelPar_KVTHOWEW,
   bool modelPar_given_KVTHOWEW,
   AdmsSensFadType & modelPar_KVTHOWELW,
   bool modelPar_given_KVTHOWELW,
   AdmsSensFadType & modelPar_KUOWEO,
   bool modelPar_given_KUOWEO,
   AdmsSensFadType & modelPar_KUOWEL,
   bool modelPar_given_KUOWEL,
   AdmsSensFadType & modelPar_KUOWEW,
   bool modelPar_given_KUOWEW,
   AdmsSensFadType & modelPar_KUOWELW,
   bool modelPar_given_KUOWELW,
   AdmsSensFadType & modelPar_LMIN,
   bool modelPar_given_LMIN,
   AdmsSensFadType & modelPar_LMAX,
   bool modelPar_given_LMAX,
   AdmsSensFadType & modelPar_WMIN,
   bool modelPar_given_WMIN,
   AdmsSensFadType & modelPar_WMAX,
   bool modelPar_given_WMAX,
   AdmsSensFadType & modelPar_RGO,
   bool modelPar_given_RGO,
   AdmsSensFadType & modelPar_RBULKO,
   bool modelPar_given_RBULKO,
   AdmsSensFadType & modelPar_RWELLO,
   bool modelPar_given_RWELLO,
   AdmsSensFadType & modelPar_RJUNSO,
   bool modelPar_given_RJUNSO,
   AdmsSensFadType & modelPar_RJUNDO,
   bool modelPar_given_RJUNDO,
   AdmsSensFadType & modelPar_RINT,
   bool modelPar_given_RINT,
   AdmsSensFadType & modelPar_RVPOLY,
   bool modelPar_given_RVPOLY,
   AdmsSensFadType & modelPar_RSHG,
   bool modelPar_given_RSHG,
   AdmsSensFadType & modelPar_DLSIL,
   bool modelPar_given_DLSIL,
   AdmsSensFadType & modelPar_SAREF,
   bool modelPar_given_SAREF,
   AdmsSensFadType & modelPar_SBREF,
   bool modelPar_given_SBREF,
   AdmsSensFadType & modelPar_WLOD,
   bool modelPar_given_WLOD,
   AdmsSensFadType & modelPar_KUO,
   bool modelPar_given_KUO,
   AdmsSensFadType & modelPar_KVSAT,
   bool modelPar_given_KVSAT,
   AdmsSensFadType & modelPar_TKUO,
   bool modelPar_given_TKUO,
   AdmsSensFadType & modelPar_LKUO,
   bool modelPar_given_LKUO,
   AdmsSensFadType & modelPar_WKUO,
   bool modelPar_given_WKUO,
   AdmsSensFadType & modelPar_PKUO,
   bool modelPar_given_PKUO,
   AdmsSensFadType & modelPar_LLODKUO,
   bool modelPar_given_LLODKUO,
   AdmsSensFadType & modelPar_WLODKUO,
   bool modelPar_given_WLODKUO,
   AdmsSensFadType & modelPar_KVTHO,
   bool modelPar_given_KVTHO,
   AdmsSensFadType & modelPar_LKVTHO,
   bool modelPar_given_LKVTHO,
   AdmsSensFadType & modelPar_WKVTHO,
   bool modelPar_given_WKVTHO,
   AdmsSensFadType & modelPar_PKVTHO,
   bool modelPar_given_PKVTHO,
   AdmsSensFadType & modelPar_LLODVTH,
   bool modelPar_given_LLODVTH,
   AdmsSensFadType & modelPar_WLODVTH,
   bool modelPar_given_WLODVTH,
   AdmsSensFadType & modelPar_STETAO,
   bool modelPar_given_STETAO,
   AdmsSensFadType & modelPar_LODETAO,
   bool modelPar_given_LODETAO,
   AdmsSensFadType & modelPar_SCREF,
   bool modelPar_given_SCREF,
   AdmsSensFadType & modelPar_WEB,
   bool modelPar_given_WEB,
   AdmsSensFadType & modelPar_WEC,
   bool modelPar_given_WEC,
   AdmsSensFadType & modelPar_IMAX,
   bool modelPar_given_IMAX,
   AdmsSensFadType & modelPar_TRJ,
   bool modelPar_given_TRJ,
   AdmsSensFadType & modelPar_CJORBOT,
   bool modelPar_given_CJORBOT,
   AdmsSensFadType & modelPar_CJORSTI,
   bool modelPar_given_CJORSTI,
   AdmsSensFadType & modelPar_CJORGAT,
   bool modelPar_given_CJORGAT,
   AdmsSensFadType & modelPar_VBIRBOT,
   bool modelPar_given_VBIRBOT,
   AdmsSensFadType & modelPar_VBIRSTI,
   bool modelPar_given_VBIRSTI,
   AdmsSensFadType & modelPar_VBIRGAT,
   bool modelPar_given_VBIRGAT,
   AdmsSensFadType & modelPar_PBOT,
   bool modelPar_given_PBOT,
   AdmsSensFadType & modelPar_PSTI,
   bool modelPar_given_PSTI,
   AdmsSensFadType & modelPar_PGAT,
   bool modelPar_given_PGAT,
   AdmsSensFadType & modelPar_PHIGBOT,
   bool modelPar_given_PHIGBOT,
   AdmsSensFadType & modelPar_PHIGSTI,
   bool modelPar_given_PHIGSTI,
   AdmsSensFadType & modelPar_PHIGGAT,
   bool modelPar_given_PHIGGAT,
   AdmsSensFadType & modelPar_IDSATRBOT,
   bool modelPar_given_IDSATRBOT,
   AdmsSensFadType & modelPar_IDSATRSTI,
   bool modelPar_given_IDSATRSTI,
   AdmsSensFadType & modelPar_IDSATRGAT,
   bool modelPar_given_IDSATRGAT,
   AdmsSensFadType & modelPar_CSRHBOT,
   bool modelPar_given_CSRHBOT,
   AdmsSensFadType & modelPar_CSRHSTI,
   bool modelPar_given_CSRHSTI,
   AdmsSensFadType & modelPar_CSRHGAT,
   bool modelPar_given_CSRHGAT,
   AdmsSensFadType & modelPar_XJUNSTI,
   bool modelPar_given_XJUNSTI,
   AdmsSensFadType & modelPar_XJUNGAT,
   bool modelPar_given_XJUNGAT,
   AdmsSensFadType & modelPar_CTATBOT,
   bool modelPar_given_CTATBOT,
   AdmsSensFadType & modelPar_CTATSTI,
   bool modelPar_given_CTATSTI,
   AdmsSensFadType & modelPar_CTATGAT,
   bool modelPar_given_CTATGAT,
   AdmsSensFadType & modelPar_MEFFTATBOT,
   bool modelPar_given_MEFFTATBOT,
   AdmsSensFadType & modelPar_MEFFTATSTI,
   bool modelPar_given_MEFFTATSTI,
   AdmsSensFadType & modelPar_MEFFTATGAT,
   bool modelPar_given_MEFFTATGAT,
   AdmsSensFadType & modelPar_CBBTBOT,
   bool modelPar_given_CBBTBOT,
   AdmsSensFadType & modelPar_CBBTSTI,
   bool modelPar_given_CBBTSTI,
   AdmsSensFadType & modelPar_CBBTGAT,
   bool modelPar_given_CBBTGAT,
   AdmsSensFadType & modelPar_FBBTRBOT,
   bool modelPar_given_FBBTRBOT,
   AdmsSensFadType & modelPar_FBBTRSTI,
   bool modelPar_given_FBBTRSTI,
   AdmsSensFadType & modelPar_FBBTRGAT,
   bool modelPar_given_FBBTRGAT,
   AdmsSensFadType & modelPar_STFBBTBOT,
   bool modelPar_given_STFBBTBOT,
   AdmsSensFadType & modelPar_STFBBTSTI,
   bool modelPar_given_STFBBTSTI,
   AdmsSensFadType & modelPar_STFBBTGAT,
   bool modelPar_given_STFBBTGAT,
   AdmsSensFadType & modelPar_VBRBOT,
   bool modelPar_given_VBRBOT,
   AdmsSensFadType & modelPar_VBRSTI,
   bool modelPar_given_VBRSTI,
   AdmsSensFadType & modelPar_VBRGAT,
   bool modelPar_given_VBRGAT,
   AdmsSensFadType & modelPar_PBRBOT,
   bool modelPar_given_PBRBOT,
   AdmsSensFadType & modelPar_PBRSTI,
   bool modelPar_given_PBRSTI,
   AdmsSensFadType & modelPar_PBRGAT,
   bool modelPar_given_PBRGAT,
   AdmsSensFadType & modelPar_CJORBOTD,
   bool modelPar_given_CJORBOTD,
   AdmsSensFadType & modelPar_CJORSTID,
   bool modelPar_given_CJORSTID,
   AdmsSensFadType & modelPar_CJORGATD,
   bool modelPar_given_CJORGATD,
   AdmsSensFadType & modelPar_VBIRBOTD,
   bool modelPar_given_VBIRBOTD,
   AdmsSensFadType & modelPar_VBIRSTID,
   bool modelPar_given_VBIRSTID,
   AdmsSensFadType & modelPar_VBIRGATD,
   bool modelPar_given_VBIRGATD,
   AdmsSensFadType & modelPar_PBOTD,
   bool modelPar_given_PBOTD,
   AdmsSensFadType & modelPar_PSTID,
   bool modelPar_given_PSTID,
   AdmsSensFadType & modelPar_PGATD,
   bool modelPar_given_PGATD,
   AdmsSensFadType & modelPar_PHIGBOTD,
   bool modelPar_given_PHIGBOTD,
   AdmsSensFadType & modelPar_PHIGSTID,
   bool modelPar_given_PHIGSTID,
   AdmsSensFadType & modelPar_PHIGGATD,
   bool modelPar_given_PHIGGATD,
   AdmsSensFadType & modelPar_IDSATRBOTD,
   bool modelPar_given_IDSATRBOTD,
   AdmsSensFadType & modelPar_IDSATRSTID,
   bool modelPar_given_IDSATRSTID,
   AdmsSensFadType & modelPar_IDSATRGATD,
   bool modelPar_given_IDSATRGATD,
   AdmsSensFadType & modelPar_CSRHBOTD,
   bool modelPar_given_CSRHBOTD,
   AdmsSensFadType & modelPar_CSRHSTID,
   bool modelPar_given_CSRHSTID,
   AdmsSensFadType & modelPar_CSRHGATD,
   bool modelPar_given_CSRHGATD,
   AdmsSensFadType & modelPar_XJUNSTID,
   bool modelPar_given_XJUNSTID,
   AdmsSensFadType & modelPar_XJUNGATD,
   bool modelPar_given_XJUNGATD,
   AdmsSensFadType & modelPar_CTATBOTD,
   bool modelPar_given_CTATBOTD,
   AdmsSensFadType & modelPar_CTATSTID,
   bool modelPar_given_CTATSTID,
   AdmsSensFadType & modelPar_CTATGATD,
   bool modelPar_given_CTATGATD,
   AdmsSensFadType & modelPar_MEFFTATBOTD,
   bool modelPar_given_MEFFTATBOTD,
   AdmsSensFadType & modelPar_MEFFTATSTID,
   bool modelPar_given_MEFFTATSTID,
   AdmsSensFadType & modelPar_MEFFTATGATD,
   bool modelPar_given_MEFFTATGATD,
   AdmsSensFadType & modelPar_CBBTBOTD,
   bool modelPar_given_CBBTBOTD,
   AdmsSensFadType & modelPar_CBBTSTID,
   bool modelPar_given_CBBTSTID,
   AdmsSensFadType & modelPar_CBBTGATD,
   bool modelPar_given_CBBTGATD,
   AdmsSensFadType & modelPar_FBBTRBOTD,
   bool modelPar_given_FBBTRBOTD,
   AdmsSensFadType & modelPar_FBBTRSTID,
   bool modelPar_given_FBBTRSTID,
   AdmsSensFadType & modelPar_FBBTRGATD,
   bool modelPar_given_FBBTRGATD,
   AdmsSensFadType & modelPar_STFBBTBOTD,
   bool modelPar_given_STFBBTBOTD,
   AdmsSensFadType & modelPar_STFBBTSTID,
   bool modelPar_given_STFBBTSTID,
   AdmsSensFadType & modelPar_STFBBTGATD,
   bool modelPar_given_STFBBTGATD,
   AdmsSensFadType & modelPar_VBRBOTD,
   bool modelPar_given_VBRBOTD,
   AdmsSensFadType & modelPar_VBRSTID,
   bool modelPar_given_VBRSTID,
   AdmsSensFadType & modelPar_VBRGATD,
   bool modelPar_given_VBRGATD,
   AdmsSensFadType & modelPar_PBRBOTD,
   bool modelPar_given_PBRBOTD,
   AdmsSensFadType & modelPar_PBRSTID,
   bool modelPar_given_PBRSTID,
   AdmsSensFadType & modelPar_PBRGATD,
   bool modelPar_given_PBRGATD,
   AdmsSensFadType & modelPar_SWJUNEXP,
   bool modelPar_given_SWJUNEXP,
   AdmsSensFadType & modelPar_VJUNREF,
   bool modelPar_given_VJUNREF,
   AdmsSensFadType & modelPar_FJUNQ,
   bool modelPar_given_FJUNQ,
   AdmsSensFadType & modelPar_VJUNREFD,
   bool modelPar_given_VJUNREFD,
   AdmsSensFadType & modelPar_FJUNQD,
   bool modelPar_given_FJUNQD// model variables
   ,
   // reals
   AdmsSensFadType & modelVar_NSUBO_i,
   AdmsSensFadType & modelVar_WSEG_i,
   AdmsSensFadType & modelVar_NPCK_i,
   AdmsSensFadType & modelVar_WSEGP_i,
   AdmsSensFadType & modelVar_LPCK_i,
   AdmsSensFadType & modelVar_LOV_i,
   AdmsSensFadType & modelVar_LOVD_i,
   AdmsSensFadType & modelVar_LP1_i,
   AdmsSensFadType & modelVar_LP2_i,
   AdmsSensFadType & modelVar_WBET_i,
   AdmsSensFadType & modelVar_AXL_i,
   AdmsSensFadType & modelVar_ALP1L2_i,
   AdmsSensFadType & modelVar_ALP2L2_i,
   AdmsSensFadType & modelVar_RINT_i,
   AdmsSensFadType & modelVar_RVPOLY_i,
   AdmsSensFadType & modelVar_SAREF_i,
   AdmsSensFadType & modelVar_SBREF_i,
   AdmsSensFadType & modelVar_KVSAT_i,
   AdmsSensFadType & modelVar_LLODKUO_i,
   AdmsSensFadType & modelVar_WLODKUO_i,
   AdmsSensFadType & modelVar_LLODVTH_i,
   AdmsSensFadType & modelVar_WLODVTH_i,
   AdmsSensFadType & modelVar_LODETAO_i,
   AdmsSensFadType & modelVar_SCREF_i,
   AdmsSensFadType & modelVar_WEB_i,
   AdmsSensFadType & modelVar_WEC_i,
   AdmsSensFadType & modelVar_TOX_i,
   AdmsSensFadType & modelVar_EPSROX_i,
   AdmsSensFadType & modelVar_TOXOV_i,
   AdmsSensFadType & modelVar_TOXOVD_i,
   AdmsSensFadType & modelVar_TKD,
   AdmsSensFadType & modelVar_dT,
   AdmsSensFadType & modelVar_rT,
   AdmsSensFadType & modelVar_rTn,
   AdmsSensFadType & modelVar_EPSSI,
   AdmsSensFadType & modelVar_phit,
   AdmsSensFadType & modelVar_inv_phit,
   AdmsSensFadType & modelVar_Eg,
   AdmsSensFadType & modelVar_phibFac,
   AdmsSensFadType & modelVar_nt0,
   AdmsSensFadType & modelVar_IMAX_i,
   AdmsSensFadType & modelVar_VBIRBOT_i,
   AdmsSensFadType & modelVar_VBIRSTI_i,
   AdmsSensFadType & modelVar_VBIRGAT_i,
   AdmsSensFadType & modelVar_PBOT_i,
   AdmsSensFadType & modelVar_PSTI_i,
   AdmsSensFadType & modelVar_PGAT_i,
   AdmsSensFadType & modelVar_CSRHBOT_i,
   AdmsSensFadType & modelVar_CSRHSTI_i,
   AdmsSensFadType & modelVar_CSRHGAT_i,
   AdmsSensFadType & modelVar_CTATBOT_i,
   AdmsSensFadType & modelVar_CTATSTI_i,
   AdmsSensFadType & modelVar_CTATGAT_i,
   AdmsSensFadType & modelVar_CBBTBOT_i,
   AdmsSensFadType & modelVar_CBBTSTI_i,
   AdmsSensFadType & modelVar_CBBTGAT_i,
   AdmsSensFadType & modelVar_VBRBOT_i,
   AdmsSensFadType & modelVar_VBRSTI_i,
   AdmsSensFadType & modelVar_VBRGAT_i,
   AdmsSensFadType & modelVar_PBRBOT_i,
   AdmsSensFadType & modelVar_PBRSTI_i,
   AdmsSensFadType & modelVar_PBRGAT_i,
   AdmsSensFadType & modelVar_SWJUNEXP_i,
   AdmsSensFadType & modelVar_VJUNREF_i,
   AdmsSensFadType & modelVar_FJUNQ_i,
   AdmsSensFadType & modelVar_phitr,
   AdmsSensFadType & modelVar_phitd,
   AdmsSensFadType & modelVar_phitdinv,
   AdmsSensFadType & modelVar_perfc,
   AdmsSensFadType & modelVar_berfc,
   AdmsSensFadType & modelVar_cerfc,
   AdmsSensFadType & modelVar_ftdbot,
   AdmsSensFadType & modelVar_ftdsti,
   AdmsSensFadType & modelVar_ftdgat,
   AdmsSensFadType & modelVar_idsatbot,
   AdmsSensFadType & modelVar_idsatsti,
   AdmsSensFadType & modelVar_idsatgat,
   AdmsSensFadType & modelVar_vbibot,
   AdmsSensFadType & modelVar_vbisti,
   AdmsSensFadType & modelVar_vbigat,
   AdmsSensFadType & modelVar_vbiinvbot,
   AdmsSensFadType & modelVar_vbiinvsti,
   AdmsSensFadType & modelVar_vbiinvgat,
   AdmsSensFadType & modelVar_one_minus_PBOT,
   AdmsSensFadType & modelVar_one_minus_PSTI,
   AdmsSensFadType & modelVar_one_minus_PGAT,
   AdmsSensFadType & modelVar_one_over_one_minus_PBOT,
   AdmsSensFadType & modelVar_one_over_one_minus_PSTI,
   AdmsSensFadType & modelVar_one_over_one_minus_PGAT,
   AdmsSensFadType & modelVar_cjobot,
   AdmsSensFadType & modelVar_cjosti,
   AdmsSensFadType & modelVar_cjogat,
   AdmsSensFadType & modelVar_qprefbot,
   AdmsSensFadType & modelVar_qprefsti,
   AdmsSensFadType & modelVar_qprefgat,
   AdmsSensFadType & modelVar_qpref2bot,
   AdmsSensFadType & modelVar_qpref2sti,
   AdmsSensFadType & modelVar_qpref2gat,
   AdmsSensFadType & modelVar_wdepnulrbot,
   AdmsSensFadType & modelVar_wdepnulrsti,
   AdmsSensFadType & modelVar_wdepnulrgat,
   AdmsSensFadType & modelVar_wdepnulrinvbot,
   AdmsSensFadType & modelVar_wdepnulrinvsti,
   AdmsSensFadType & modelVar_wdepnulrinvgat,
   AdmsSensFadType & modelVar_VBIRBOTinv,
   AdmsSensFadType & modelVar_VBIRSTIinv,
   AdmsSensFadType & modelVar_VBIRGATinv,
   AdmsSensFadType & modelVar_atatbot,
   AdmsSensFadType & modelVar_atatsti,
   AdmsSensFadType & modelVar_atatgat,
   AdmsSensFadType & modelVar_btatpartbot,
   AdmsSensFadType & modelVar_btatpartsti,
   AdmsSensFadType & modelVar_btatpartgat,
   AdmsSensFadType & modelVar_fbbtbot,
   AdmsSensFadType & modelVar_fbbtsti,
   AdmsSensFadType & modelVar_fbbtgat,
   AdmsSensFadType & modelVar_fstopbot,
   AdmsSensFadType & modelVar_fstopsti,
   AdmsSensFadType & modelVar_fstopgat,
   AdmsSensFadType & modelVar_VBRinvbot,
   AdmsSensFadType & modelVar_VBRinvsti,
   AdmsSensFadType & modelVar_VBRinvgat,
   AdmsSensFadType & modelVar_slopebot,
   AdmsSensFadType & modelVar_slopesti,
   AdmsSensFadType & modelVar_slopegat,
   AdmsSensFadType & modelVar_VBIRBOTD_i,
   AdmsSensFadType & modelVar_VBIRSTID_i,
   AdmsSensFadType & modelVar_VBIRGATD_i,
   AdmsSensFadType & modelVar_PBOTD_i,
   AdmsSensFadType & modelVar_PSTID_i,
   AdmsSensFadType & modelVar_PGATD_i,
   AdmsSensFadType & modelVar_CSRHBOTD_i,
   AdmsSensFadType & modelVar_CSRHSTID_i,
   AdmsSensFadType & modelVar_CSRHGATD_i,
   AdmsSensFadType & modelVar_CTATBOTD_i,
   AdmsSensFadType & modelVar_CTATSTID_i,
   AdmsSensFadType & modelVar_CTATGATD_i,
   AdmsSensFadType & modelVar_CBBTBOTD_i,
   AdmsSensFadType & modelVar_CBBTSTID_i,
   AdmsSensFadType & modelVar_CBBTGATD_i,
   AdmsSensFadType & modelVar_VBRBOTD_i,
   AdmsSensFadType & modelVar_VBRSTID_i,
   AdmsSensFadType & modelVar_VBRGATD_i,
   AdmsSensFadType & modelVar_PBRBOTD_i,
   AdmsSensFadType & modelVar_PBRSTID_i,
   AdmsSensFadType & modelVar_PBRGATD_i,
   AdmsSensFadType & modelVar_VJUNREFD_i,
   AdmsSensFadType & modelVar_FJUNQD_i,
   AdmsSensFadType & modelVar_ftdbot_d,
   AdmsSensFadType & modelVar_ftdsti_d,
   AdmsSensFadType & modelVar_ftdgat_d,
   AdmsSensFadType & modelVar_idsatbot_d,
   AdmsSensFadType & modelVar_idsatsti_d,
   AdmsSensFadType & modelVar_idsatgat_d,
   AdmsSensFadType & modelVar_vbibot_d,
   AdmsSensFadType & modelVar_vbisti_d,
   AdmsSensFadType & modelVar_vbigat_d,
   AdmsSensFadType & modelVar_vbiinvbot_d,
   AdmsSensFadType & modelVar_vbiinvsti_d,
   AdmsSensFadType & modelVar_vbiinvgat_d,
   AdmsSensFadType & modelVar_one_minus_PBOT_d,
   AdmsSensFadType & modelVar_one_minus_PSTI_d,
   AdmsSensFadType & modelVar_one_minus_PGAT_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PBOT_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PSTI_d,
   AdmsSensFadType & modelVar_one_over_one_minus_PGAT_d,
   AdmsSensFadType & modelVar_cjobot_d,
   AdmsSensFadType & modelVar_cjosti_d,
   AdmsSensFadType & modelVar_cjogat_d,
   AdmsSensFadType & modelVar_qprefbot_d,
   AdmsSensFadType & modelVar_qprefsti_d,
   AdmsSensFadType & modelVar_qprefgat_d,
   AdmsSensFadType & modelVar_qpref2bot_d,
   AdmsSensFadType & modelVar_qpref2sti_d,
   AdmsSensFadType & modelVar_qpref2gat_d,
   AdmsSensFadType & modelVar_wdepnulrbot_d,
   AdmsSensFadType & modelVar_wdepnulrsti_d,
   AdmsSensFadType & modelVar_wdepnulrgat_d,
   AdmsSensFadType & modelVar_wdepnulrinvbot_d,
   AdmsSensFadType & modelVar_wdepnulrinvsti_d,
   AdmsSensFadType & modelVar_wdepnulrinvgat_d,
   AdmsSensFadType & modelVar_VBIRBOTinv_d,
   AdmsSensFadType & modelVar_VBIRSTIinv_d,
   AdmsSensFadType & modelVar_VBIRGATinv_d,
   AdmsSensFadType & modelVar_atatbot_d,
   AdmsSensFadType & modelVar_atatsti_d,
   AdmsSensFadType & modelVar_atatgat_d,
   AdmsSensFadType & modelVar_btatpartbot_d,
   AdmsSensFadType & modelVar_btatpartsti_d,
   AdmsSensFadType & modelVar_btatpartgat_d,
   AdmsSensFadType & modelVar_fbbtbot_d,
   AdmsSensFadType & modelVar_fbbtsti_d,
   AdmsSensFadType & modelVar_fbbtgat_d,
   AdmsSensFadType & modelVar_fstopbot_d,
   AdmsSensFadType & modelVar_fstopsti_d,
   AdmsSensFadType & modelVar_fstopgat_d,
   AdmsSensFadType & modelVar_VBRinvbot_d,
   AdmsSensFadType & modelVar_VBRinvsti_d,
   AdmsSensFadType & modelVar_VBRinvgat_d,
   AdmsSensFadType & modelVar_slopebot_d,
   AdmsSensFadType & modelVar_slopesti_d,
   AdmsSensFadType & modelVar_slopegat_d,
   // non-reals
   int modelVar_CHNL_TYPE,
   double admsTemperature, double ADMSgmin_arg, const Instance & theInstance);

#endif // Xyce_ADMS_SENSITIVITIES


// Limited exponential --- NOT what verilog LRM says, but what qucs,
// ng-spice, and zspice do.

template <typename T>
T limexp(const T &x)
{
  if ((x) < 80.0)
    return (exp(x));
  else
    return (exp(80.0)*(x-79.0));
}

// Maybe this will do as a substitute for a sacado-provided "floor" and "ceil"?
// It is wrong to try to propagate the Sacado derivatives to the return
// value, as the floor and ceil functions have zero derivative w.r.t. any
// of the variables in the argument, except when X is an integer at which
// point the derivative is undefined.
template <typename T>
double ADMSfloor(const T & x)
{
  return (floor(x.val()));
}

template <typename T>
double ADMSceil(const T & x)
{
  return (ceil(x.val()));
}


struct Traits: public DeviceTraits<Model, Instance, MOSFET1::Traits>
{
  static const char *name() {return "PSP102VA legacy MOSFET 102.5";}
  static const char *deviceTypeName() {return "m level 102";}

  static int numNodes() {return 4;}
  static bool modelRequired() {return true;}
  static bool isLinearDevice() {return false;}

  static Device *factory(const Configuration &configuration, const FactoryBlock &factory_block);
  static void loadModelParameters(ParametricData<Model> &model_parameters);
  static void loadInstanceParameters(ParametricData<Instance> &instance_parameters);
};

//-----------------------------------------------------------------------------
// Class         : Instance

//
// Purpose       : This class represents a single instance  of the
//                 device.  It mainly contains indices and pointers into
//                 the matrix equation (see the resistor instance class for
//                 more details).
//
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class Instance : public DeviceInstance
{
  friend class ParametricData<Instance>;
  friend class Model;
#ifdef Xyce_ADMS_SENSITIVITIES
  friend class InstanceSensitivity;
  friend class ModelSensitivity;
#endif // Xyce_ADMS_SENSITIVITIES
  friend struct Traits;

public:
  Instance(
     const Configuration &       configuration,
     const InstanceBlock &       instance_block,
     Model &                     model,
     const FactoryBlock &        factory_block);

  ~Instance();

private:
  Instance(const Instance &);
  Instance &operator=(const Instance &);

public:
  void registerLIDs( const LocalIdVector & intLIDVecRef,
                     const LocalIdVector & extLIDVecRef );
  void registerStoreLIDs( const LocalIdVector & stoLIDVecRef );
  void setupPointers();

  void loadNodeSymbols(Util::SymbolTable &symbol_table) const;

  const JacobianStamp & jacobianStamp() const;
  void registerJacLIDs( const JacobianStamp & jacLIDVec );

  void registerBranchDataLIDs(const std::vector<int> & branchLIDVecRef);

  bool processParams();
  bool updateTemperature ( const double & temp = -999.0 );
  bool updateIntermediateVars ();
  bool updatePrimaryState ();
  bool updateSecondaryState ();

  // load functions, residual:
  bool loadDAEQVector ();
  bool loadDAEFVector ();

  // load functions, Jacobian:
  bool loadDAEdQdx ();
  bool loadDAEdFdx ();

  void collapseNodes();
  int getNumNoiseSources () const;  // 12
  void setupNoiseSources (Xyce::Analysis::NoiseData & noiseData);
  void getNoiseSources (Xyce::Analysis::NoiseData & noiseData);

private:

public:
  // iterator reference to the PSP102VA model which owns this instance.
  // Getters and setters
  Model &getModel()
  {
    return model_;
  }

private:

  Model & model_;   //< Owning Model
  // Begin verilog Instance Variables
  //   Instance Parameters
  double L;
  double W;
  double SA;
  double SB;
  double SD;
  double SCA;
  double SCB;
  double SCC;
  double SC;
  double DELVTO;
  double FACTUO;
  double ABSOURCE;
  double LSSOURCE;
  double LGSOURCE;
  double ABDRAIN;
  double LSDRAIN;
  double LGDRAIN;
  double AS;
  double PS;
  double AD;
  double PD;
  double MULT;
  double NF;
  double NGCON;
  double XGW;
  //  Variables of global_instance scope
  double VFB_i;
  double VNSUB_i;
  double NSLP_i;
  double DNSUB_i;
  double CF_i;
  double CFB_i;
  double MUE_i;
  double THEMU_i;
  double CS_i;
  double XCOR_i;
  double THER_i;
  double RSB_i;
  double RSG_i;
  double THESAT_i;
  double THESATB_i;
  double THESATG_i;
  double AX_i;
  double ALP_i;
  double ALP1_i;
  double ALP2_i;
  double A1_i;
  double A2_i;
  double A3_i;
  double A4_i;
  double IGINV_i;
  double IGOV_i;
  double IGOVD_i;
  double GC2_i;
  double GC3_i;
  double CHIB_i;
  double AGIDL_i;
  double CGIDL_i;
  double AGIDLD_i;
  double CGIDLD_i;
  double COX_i;
  double CGOV_i;
  double CGOVD_i;
  double CGBOV_i;
  double CFR_i;
  double CFRD_i;
  double NFA_i;
  double NFB_i;
  double NFC_i;
  double EF_i;
  double FNTEXC_i;
  double MULT_i;
  double BET_i;
  double kp;
  double qq;
  double qlim2;
  double E_eff0;
  double eta_mu;
  double BCH;
  double BOV;
  double inv_CHIB;
  double GCQ;
  double Dch;
  double Dov;
  double x1;
  double Vdsat_lim;
  double nt;
  double Cox_over_q;
  double GOV_s;
  double GOV_d;
  double GOV2_s;
  double GOV2_d;
  double BOV_d;
  double xi_ov_s;
  double xi_ov_d;
  double inv_xi_ov_s;
  double inv_xi_ov_d;
  double x_mrg_ov_s;
  double x_mrg_ov_d;
  double inv_xg1_s;
  double inv_xg1_d;
  double AGIDLs;
  double AGIDLDs;
  double BGIDLs;
  double BGIDLDs;
  double phib;
  double sqrt_phib;
  double phix;
  double aphi;
  double bphi;
  double phix1;
  double G_0;
  double phit1;
  double inv_phit1;
  double alpha_b;
  double inv_VP;
  double inv_AX;
  double Sfl_prefac;
  double fac_exc;
  double RG_i;
  double RBULK_i;
  double RWELL_i;
  double RJUNS_i;
  double RJUND_i;
  double ggate;
  double gbulk;
  double gwell;
  double gjund;
  double gjuns;
  double ABSOURCE_i;
  double LSSOURCE_i;
  double LGSOURCE_i;
  double zflagbot_s;
  double zflagsti_s;
  double zflaggat_s;
  double VMAX_s;
  double exp_VMAX_over_phitd_s;
  double vbimin_s;
  double vch_s;
  double vfmin_s;
  double vbbtlim_s;
  double xhighf1_s;
  double expxhf1_s;
  double xhighf2_s;
  double expxhf2_s;
  double xhighr_s;
  double expxhr_s;
  double m0flag_s;
  double ISATFOR1_s;
  double MFOR1_s;
  double ISATFOR2_s;
  double MFOR2_s;
  double ISATREV_s;
  double MREV_s;
  double ABDRAIN_i;
  double LSDRAIN_i;
  double LGDRAIN_i;
  double zflagbot_d;
  double zflagsti_d;
  double zflaggat_d;
  double VMAX_d;
  double exp_VMAX_over_phitd_d;
  double vbimin_d;
  double vch_d;
  double vfmin_d;
  double vbbtlim_d;
  double xhighf1_d;
  double expxhf1_d;
  double xhighf2_d;
  double expxhf2_d;
  double xhighr_d;
  double expxhr_d;
  double m0flag_d;
  double ISATFOR1_d;
  double MFOR1_d;
  double ISATFOR2_d;
  double MFOR2_d;
  double ISATREV_d;
  double MREV_d;
  // end verilog Instance Variables=====
  // Nodal LID Variables
  int li_D;
  int li_G;
  int li_S;
  int li_B;
  int li_NOI;
  int li_NOI2;
  int li_GP;
  int li_BP;
  int li_BI;
  int li_BS;
  int li_BD;
  // end Nodal LID Variables
  // Branch LID Variables
  // end Branch LID Variables
  // Lead (branch) LID Variables
  int li_branch_iD;
  int li_branch_iG;
  int li_branch_iS;
  int li_branch_iB;
  // end Lead (branch) LID Variables
  // Jacobian  pointers
  double * f_D_Equ_S_Node_Ptr;
  double * f_D_Equ_BP_Node_Ptr;
  double * f_BP_Equ_S_Node_Ptr;
  double * f_BP_Equ_BP_Node_Ptr;
  double * f_D_Equ_D_Node_Ptr;
  double * f_BP_Equ_D_Node_Ptr;
  double * f_D_Equ_GP_Node_Ptr;
  double * f_BP_Equ_GP_Node_Ptr;
  double * f_S_Equ_S_Node_Ptr;
  double * f_S_Equ_BP_Node_Ptr;
  double * f_S_Equ_D_Node_Ptr;
  double * f_S_Equ_GP_Node_Ptr;
  double * f_GP_Equ_S_Node_Ptr;
  double * f_GP_Equ_BP_Node_Ptr;
  double * f_GP_Equ_D_Node_Ptr;
  double * f_GP_Equ_GP_Node_Ptr;
  double * f_BS_Equ_S_Node_Ptr;
  double * f_BS_Equ_BS_Node_Ptr;
  double * f_S_Equ_BS_Node_Ptr;
  double * f_BS_Equ_D_Node_Ptr;
  double * f_BS_Equ_BD_Node_Ptr;
  double * f_S_Equ_BD_Node_Ptr;
  double * f_BD_Equ_S_Node_Ptr;
  double * f_BD_Equ_BS_Node_Ptr;
  double * f_D_Equ_BS_Node_Ptr;
  double * f_BD_Equ_D_Node_Ptr;
  double * f_BD_Equ_BD_Node_Ptr;
  double * f_D_Equ_BD_Node_Ptr;
  double * f_G_Equ_G_Node_Ptr;
  double * f_G_Equ_GP_Node_Ptr;
  double * f_GP_Equ_G_Node_Ptr;
  double * f_BP_Equ_BI_Node_Ptr;
  double * f_BI_Equ_BP_Node_Ptr;
  double * f_BI_Equ_BI_Node_Ptr;
  double * f_BS_Equ_BI_Node_Ptr;
  double * f_BI_Equ_BS_Node_Ptr;
  double * f_BD_Equ_BI_Node_Ptr;
  double * f_BI_Equ_BD_Node_Ptr;
  double * f_B_Equ_B_Node_Ptr;
  double * f_B_Equ_BI_Node_Ptr;
  double * f_BI_Equ_B_Node_Ptr;
  double * f_NOI2_Equ_NOI2_Node_Ptr;
  double * f_NOI2_Equ_S_Node_Ptr;
  double * f_NOI2_Equ_BP_Node_Ptr;
  double * f_NOI2_Equ_GP_Node_Ptr;
  double * f_NOI2_Equ_D_Node_Ptr;
  double * f_NOI_Equ_S_Node_Ptr;
  double * f_NOI_Equ_BP_Node_Ptr;
  double * f_NOI_Equ_GP_Node_Ptr;
  double * f_NOI_Equ_D_Node_Ptr;
  double * f_NOI_Equ_NOI2_Node_Ptr;
  double * f_NOI_Equ_NOI_Node_Ptr;
  double * f_D_Equ_NOI2_Node_Ptr;
  double * f_S_Equ_NOI2_Node_Ptr;
  double * f_GP_Equ_NOI_Node_Ptr;
  double * f_S_Equ_NOI_Node_Ptr;
  double * f_D_Equ_NOI_Node_Ptr;
  double * f_BS_Equ_BP_Node_Ptr;
  double * f_BS_Equ_GP_Node_Ptr;
  double * f_BD_Equ_BP_Node_Ptr;
  double * f_BD_Equ_GP_Node_Ptr;
  double * q_D_Equ_S_Node_Ptr;
  double * q_D_Equ_BP_Node_Ptr;
  double * q_BP_Equ_S_Node_Ptr;
  double * q_BP_Equ_BP_Node_Ptr;
  double * q_D_Equ_D_Node_Ptr;
  double * q_BP_Equ_D_Node_Ptr;
  double * q_D_Equ_GP_Node_Ptr;
  double * q_BP_Equ_GP_Node_Ptr;
  double * q_S_Equ_S_Node_Ptr;
  double * q_S_Equ_BP_Node_Ptr;
  double * q_S_Equ_D_Node_Ptr;
  double * q_S_Equ_GP_Node_Ptr;
  double * q_GP_Equ_S_Node_Ptr;
  double * q_GP_Equ_BP_Node_Ptr;
  double * q_GP_Equ_D_Node_Ptr;
  double * q_GP_Equ_GP_Node_Ptr;
  double * q_BS_Equ_S_Node_Ptr;
  double * q_BS_Equ_BS_Node_Ptr;
  double * q_S_Equ_BS_Node_Ptr;
  double * q_BS_Equ_D_Node_Ptr;
  double * q_BS_Equ_BD_Node_Ptr;
  double * q_S_Equ_BD_Node_Ptr;
  double * q_BD_Equ_S_Node_Ptr;
  double * q_BD_Equ_BS_Node_Ptr;
  double * q_D_Equ_BS_Node_Ptr;
  double * q_BD_Equ_D_Node_Ptr;
  double * q_BD_Equ_BD_Node_Ptr;
  double * q_D_Equ_BD_Node_Ptr;
  double * q_G_Equ_G_Node_Ptr;
  double * q_G_Equ_GP_Node_Ptr;
  double * q_GP_Equ_G_Node_Ptr;
  double * q_BP_Equ_BI_Node_Ptr;
  double * q_BI_Equ_BP_Node_Ptr;
  double * q_BI_Equ_BI_Node_Ptr;
  double * q_BS_Equ_BI_Node_Ptr;
  double * q_BI_Equ_BS_Node_Ptr;
  double * q_BD_Equ_BI_Node_Ptr;
  double * q_BI_Equ_BD_Node_Ptr;
  double * q_B_Equ_B_Node_Ptr;
  double * q_B_Equ_BI_Node_Ptr;
  double * q_BI_Equ_B_Node_Ptr;
  double * q_NOI2_Equ_NOI2_Node_Ptr;
  double * q_NOI2_Equ_S_Node_Ptr;
  double * q_NOI2_Equ_BP_Node_Ptr;
  double * q_NOI2_Equ_GP_Node_Ptr;
  double * q_NOI2_Equ_D_Node_Ptr;
  double * q_NOI_Equ_S_Node_Ptr;
  double * q_NOI_Equ_BP_Node_Ptr;
  double * q_NOI_Equ_GP_Node_Ptr;
  double * q_NOI_Equ_D_Node_Ptr;
  double * q_NOI_Equ_NOI2_Node_Ptr;
  double * q_NOI_Equ_NOI_Node_Ptr;
  double * q_D_Equ_NOI2_Node_Ptr;
  double * q_S_Equ_NOI2_Node_Ptr;
  double * q_GP_Equ_NOI_Node_Ptr;
  double * q_S_Equ_NOI_Node_Ptr;
  double * q_D_Equ_NOI_Node_Ptr;
  double * q_BS_Equ_BP_Node_Ptr;
  double * q_BS_Equ_GP_Node_Ptr;
  double * q_BD_Equ_BP_Node_Ptr;
  double * q_BD_Equ_GP_Node_Ptr;
  // Jacobian offsets
  int A_D_Equ_S_NodeOffset;
  int A_D_Equ_BP_NodeOffset;
  int A_BP_Equ_S_NodeOffset;
  int A_BP_Equ_BP_NodeOffset;
  int A_D_Equ_D_NodeOffset;
  int A_BP_Equ_D_NodeOffset;
  int A_D_Equ_GP_NodeOffset;
  int A_BP_Equ_GP_NodeOffset;
  int A_S_Equ_S_NodeOffset;
  int A_S_Equ_BP_NodeOffset;
  int A_S_Equ_D_NodeOffset;
  int A_S_Equ_GP_NodeOffset;
  int A_GP_Equ_S_NodeOffset;
  int A_GP_Equ_BP_NodeOffset;
  int A_GP_Equ_D_NodeOffset;
  int A_GP_Equ_GP_NodeOffset;
  int A_BS_Equ_S_NodeOffset;
  int A_BS_Equ_BS_NodeOffset;
  int A_S_Equ_BS_NodeOffset;
  int A_BS_Equ_D_NodeOffset;
  int A_BS_Equ_BD_NodeOffset;
  int A_S_Equ_BD_NodeOffset;
  int A_BD_Equ_S_NodeOffset;
  int A_BD_Equ_BS_NodeOffset;
  int A_D_Equ_BS_NodeOffset;
  int A_BD_Equ_D_NodeOffset;
  int A_BD_Equ_BD_NodeOffset;
  int A_D_Equ_BD_NodeOffset;
  int A_G_Equ_G_NodeOffset;
  int A_G_Equ_GP_NodeOffset;
  int A_GP_Equ_G_NodeOffset;
  int A_BP_Equ_BI_NodeOffset;
  int A_BI_Equ_BP_NodeOffset;
  int A_BI_Equ_BI_NodeOffset;
  int A_BS_Equ_BI_NodeOffset;
  int A_BI_Equ_BS_NodeOffset;
  int A_BD_Equ_BI_NodeOffset;
  int A_BI_Equ_BD_NodeOffset;
  int A_B_Equ_B_NodeOffset;
  int A_B_Equ_BI_NodeOffset;
  int A_BI_Equ_B_NodeOffset;
  int A_NOI2_Equ_NOI2_NodeOffset;
  int A_NOI2_Equ_S_NodeOffset;
  int A_NOI2_Equ_BP_NodeOffset;
  int A_NOI2_Equ_GP_NodeOffset;
  int A_NOI2_Equ_D_NodeOffset;
  int A_NOI_Equ_S_NodeOffset;
  int A_NOI_Equ_BP_NodeOffset;
  int A_NOI_Equ_GP_NodeOffset;
  int A_NOI_Equ_D_NodeOffset;
  int A_NOI_Equ_NOI2_NodeOffset;
  int A_NOI_Equ_NOI_NodeOffset;
  int A_D_Equ_NOI2_NodeOffset;
  int A_S_Equ_NOI2_NodeOffset;
  int A_GP_Equ_NOI_NodeOffset;
  int A_S_Equ_NOI_NodeOffset;
  int A_D_Equ_NOI_NodeOffset;
  int A_BS_Equ_BP_NodeOffset;
  int A_BS_Equ_GP_NodeOffset;
  int A_BD_Equ_BP_NodeOffset;
  int A_BD_Equ_GP_NodeOffset;
  // end of Jacobian and pointers
  // node numbers
  static const int admsNodeID_D = 0;
  static const int admsNodeID_G = 1;
  static const int admsNodeID_S = 2;
  static const int admsNodeID_B = 3;
  static const int admsNodeID_NOI = 4;
  static const int admsNodeID_NOI2 = 5;
  static const int admsNodeID_GP = 6;
  static const int admsNodeID_BP = 7;
  static const int admsNodeID_BI = 8;
  static const int admsNodeID_BS = 9;
  static const int admsNodeID_BD = 10;
  static const int admsNodeID_GND = -1;
  // end node numbers
  // Additional IDs for branch equations
  // end branch numbers
  // Probe numbers
  static const int admsProbeID_V_NOI_GND = 0;
  static const int admsProbeID_V_NOI2_GND = 1;
  static const int admsProbeID_V_D_BD = 2;
  static const int admsProbeID_V_S_BS = 3;
  static const int admsProbeID_V_S_BP = 4;
  static const int admsProbeID_V_D_S = 5;
  static const int admsProbeID_V_GP_S = 6;
  static const int admsProbeID_V_B_BI = 7;
  static const int admsProbeID_V_BP_BI = 8;
  static const int admsProbeID_V_BD_BI = 9;
  static const int admsProbeID_V_BS_BI = 10;
  static const int admsProbeID_V_G_GP = 11;
  // end probe numbers
  // Store LIDs
  // end store LIDs
  // bools for collapsing nodes
  bool collapseNode_GP;
  bool collapseNode_BP;
  bool collapseNode_BI;
  bool collapseNode_BS;
  bool collapseNode_BD;
  // Arrays to hold probes
  std::vector < double > probeVars;
  std::vector < std::vector < double > > d_probeVars;
  // Arrays to hold contributions
  // dynamic contributions are differentiated w.r.t time
  std::vector < double > staticContributions;
  std::vector < std::vector < double > > d_staticContributions;
  std::vector < double > dynamicContributions;
  std::vector < std::vector < double > > d_dynamicContributions;
  //vectors to hold noise information for communication between methods
  std::vector<double> noiseContribsPower;
  std::vector<double> noiseContribsExponent;


  // this is what we'll use when any model uses $temperature.  We'll
  // set it in updateTemperature, and initialize it to whatever
  // is in devOptions when the instance is constructed.
  double admsTemperature;

  // vt at $temperature;
  double adms_vt_nom;


  // This one is for the annoying bogus "XyceADMSInstTemp" parameter
  // that we need so we can set it from the device manager when there's no
  // "TEMP" parameter to use
  double admsInstTemp;


  JacobianStamp jacStamp;
  IdVector nodeMap;
  PairMap pairToJacStampMap;

  // These instance-owned vectors are for storage of lead current data
  std::vector<double> leadCurrentF;
  std::vector<double> leadCurrentQ;
};


//-----------------------------------------------------------------------------
// Class         : Model

// Purpose       :
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class Model : public DeviceModel
{
  typedef std::vector<Instance *> InstanceVector;

  friend class ParametricData<Model>;
  friend class Instance;
#ifdef Xyce_ADMS_SENSITIVITIES
  friend class InstanceSensitivity;
  friend class ModelSensitivity;
#endif // Xyce_ADMS_SENSITIVITIES
  friend struct Traits;

public:
  Model(
     const Configuration &       configuration,
     const ModelBlock &          model_block,
     const FactoryBlock &        factory_block);

  ~Model();

private:
  Model(const Model &);
  Model &operator=(const Model &);

public:
  virtual void forEachInstance(DeviceInstanceOp &op) const /* override */;
  virtual std::ostream &printOutInstances(std::ostream &os) const;
  bool processParams();
  bool processInstanceParams();

private:

public:
  void addInstance(Instance *instance)
  {
    instanceContainer.push_back(instance);
  }

  void setupBaseInstanceContainer()
  {
    std::vector<Instance*>::iterator iter = instanceContainer.begin();
    std::vector<Instance*>::iterator end   = instanceContainer.end();
    for ( ; iter!=end; ++iter)
    {
      Xyce::Device::DeviceModel::baseInstanceContainer.push_back( static_cast<Xyce::Device::DeviceInstance *>(*iter) );
    }
  }

private:
  std::vector<Instance*> instanceContainer;

private:

  // This one is for the annoying bogus "XyceADMSInstTemp" parameter
  // that we need so we can set it from the device manager when there's no
  // "TEMP" model parameter to use
  double admsModTemp;
  // Begin verilog Model Variables
  //   Model Parameters
  double LEVEL;
  double TYPE;
  double TR;
  double SWIGATE;
  double SWIMPACT;
  double SWGIDL;
  double SWJUNCAP;
  double SWJUNASYM;
  double QMC;
  double LVARO;
  double LVARL;
  double LVARW;
  double LAP;
  double WVARO;
  double WVARL;
  double WVARW;
  double WOT;
  double DLQ;
  double DWQ;
  double VFBO;
  double VFBL;
  double VFBW;
  double VFBLW;
  double STVFBO;
  double STVFBL;
  double STVFBW;
  double STVFBLW;
  double TOXO;
  double EPSROXO;
  double NSUBO;
  double NSUBW;
  double WSEG;
  double NPCK;
  double NPCKW;
  double WSEGP;
  double LPCK;
  double LPCKW;
  double FOL1;
  double FOL2;
  double VNSUBO;
  double NSLPO;
  double DNSUBO;
  double DPHIBO;
  double DPHIBL;
  double DPHIBLEXP;
  double DPHIBW;
  double DPHIBLW;
  double NPO;
  double NPL;
  double CTO;
  double CTL;
  double CTLEXP;
  double CTW;
  double CTLW;
  double TOXOVO;
  double TOXOVDO;
  double LOV;
  double LOVD;
  double NOVO;
  double NOVDO;
  double CFL;
  double CFLEXP;
  double CFW;
  double CFBO;
  double UO;
  double FBET1;
  double FBET1W;
  double LP1;
  double LP1W;
  double FBET2;
  double LP2;
  double BETW1;
  double BETW2;
  double WBET;
  double STBETO;
  double STBETL;
  double STBETW;
  double STBETLW;
  double MUEO;
  double MUEW;
  double STMUEO;
  double THEMUO;
  double STTHEMUO;
  double CSO;
  double CSL;
  double CSLEXP;
  double CSW;
  double CSLW;
  double STCSO;
  double XCORO;
  double XCORL;
  double XCORW;
  double XCORLW;
  double STXCORO;
  double FETAO;
  double RSW1;
  double RSW2;
  double STRSO;
  double RSBO;
  double RSGO;
  double THESATO;
  double THESATL;
  double THESATLEXP;
  double THESATW;
  double THESATLW;
  double STTHESATO;
  double STTHESATL;
  double STTHESATW;
  double STTHESATLW;
  double THESATBO;
  double THESATGO;
  double AXO;
  double AXL;
  double ALPL;
  double ALPLEXP;
  double ALPW;
  double ALP1L1;
  double ALP1LEXP;
  double ALP1L2;
  double ALP1W;
  double ALP2L1;
  double ALP2LEXP;
  double ALP2L2;
  double ALP2W;
  double VPO;
  double A1O;
  double A1L;
  double A1W;
  double A2O;
  double STA2O;
  double A3O;
  double A3L;
  double A3W;
  double A4O;
  double A4L;
  double A4W;
  double GCOO;
  double IGINVLW;
  double IGOVW;
  double IGOVDW;
  double STIGO;
  double GC2O;
  double GC3O;
  double CHIBO;
  double AGIDLW;
  double AGIDLDW;
  double BGIDLO;
  double BGIDLDO;
  double STBGIDLO;
  double STBGIDLDO;
  double CGIDLO;
  double CGIDLDO;
  double CGBOVL;
  double CFRW;
  double CFRDW;
  double FNTO;
  double FNTEXCL;
  double NFALW;
  double NFBLW;
  double NFCLW;
  double EFO;
  double LINTNOI;
  double ALPNOI;
  double DTA;
  double KVTHOWEO;
  double KVTHOWEL;
  double KVTHOWEW;
  double KVTHOWELW;
  double KUOWEO;
  double KUOWEL;
  double KUOWEW;
  double KUOWELW;
  double LMIN;
  double LMAX;
  double WMIN;
  double WMAX;
  double RGO;
  double RBULKO;
  double RWELLO;
  double RJUNSO;
  double RJUNDO;
  double RINT;
  double RVPOLY;
  double RSHG;
  double DLSIL;
  double SAREF;
  double SBREF;
  double WLOD;
  double KUO;
  double KVSAT;
  double TKUO;
  double LKUO;
  double WKUO;
  double PKUO;
  double LLODKUO;
  double WLODKUO;
  double KVTHO;
  double LKVTHO;
  double WKVTHO;
  double PKVTHO;
  double LLODVTH;
  double WLODVTH;
  double STETAO;
  double LODETAO;
  double SCREF;
  double WEB;
  double WEC;
  double IMAX;
  double TRJ;
  double CJORBOT;
  double CJORSTI;
  double CJORGAT;
  double VBIRBOT;
  double VBIRSTI;
  double VBIRGAT;
  double PBOT;
  double PSTI;
  double PGAT;
  double PHIGBOT;
  double PHIGSTI;
  double PHIGGAT;
  double IDSATRBOT;
  double IDSATRSTI;
  double IDSATRGAT;
  double CSRHBOT;
  double CSRHSTI;
  double CSRHGAT;
  double XJUNSTI;
  double XJUNGAT;
  double CTATBOT;
  double CTATSTI;
  double CTATGAT;
  double MEFFTATBOT;
  double MEFFTATSTI;
  double MEFFTATGAT;
  double CBBTBOT;
  double CBBTSTI;
  double CBBTGAT;
  double FBBTRBOT;
  double FBBTRSTI;
  double FBBTRGAT;
  double STFBBTBOT;
  double STFBBTSTI;
  double STFBBTGAT;
  double VBRBOT;
  double VBRSTI;
  double VBRGAT;
  double PBRBOT;
  double PBRSTI;
  double PBRGAT;
  double CJORBOTD;
  double CJORSTID;
  double CJORGATD;
  double VBIRBOTD;
  double VBIRSTID;
  double VBIRGATD;
  double PBOTD;
  double PSTID;
  double PGATD;
  double PHIGBOTD;
  double PHIGSTID;
  double PHIGGATD;
  double IDSATRBOTD;
  double IDSATRSTID;
  double IDSATRGATD;
  double CSRHBOTD;
  double CSRHSTID;
  double CSRHGATD;
  double XJUNSTID;
  double XJUNGATD;
  double CTATBOTD;
  double CTATSTID;
  double CTATGATD;
  double MEFFTATBOTD;
  double MEFFTATSTID;
  double MEFFTATGATD;
  double CBBTBOTD;
  double CBBTSTID;
  double CBBTGATD;
  double FBBTRBOTD;
  double FBBTRSTID;
  double FBBTRGATD;
  double STFBBTBOTD;
  double STFBBTSTID;
  double STFBBTGATD;
  double VBRBOTD;
  double VBRSTID;
  double VBRGATD;
  double PBRBOTD;
  double PBRSTID;
  double PBRGATD;
  double SWJUNEXP;
  double VJUNREF;
  double FJUNQ;
  double VJUNREFD;
  double FJUNQD;
  //  Variables of global_model scope
  double NSUBO_i;
  double WSEG_i;
  double NPCK_i;
  double WSEGP_i;
  double LPCK_i;
  double LOV_i;
  double LOVD_i;
  double LP1_i;
  double LP2_i;
  double WBET_i;
  double AXL_i;
  double ALP1L2_i;
  double ALP2L2_i;
  double RINT_i;
  double RVPOLY_i;
  double SAREF_i;
  double SBREF_i;
  double KVSAT_i;
  double LLODKUO_i;
  double WLODKUO_i;
  double LLODVTH_i;
  double WLODVTH_i;
  double LODETAO_i;
  double SCREF_i;
  double WEB_i;
  double WEC_i;
  double TOX_i;
  double EPSROX_i;
  double TOXOV_i;
  double TOXOVD_i;
  double TKD;
  double dT;
  double rT;
  double rTn;
  double EPSSI;
  double phit;
  double inv_phit;
  double Eg;
  double phibFac;
  double nt0;
  int CHNL_TYPE;
  double IMAX_i;
  double VBIRBOT_i;
  double VBIRSTI_i;
  double VBIRGAT_i;
  double PBOT_i;
  double PSTI_i;
  double PGAT_i;
  double CSRHBOT_i;
  double CSRHSTI_i;
  double CSRHGAT_i;
  double CTATBOT_i;
  double CTATSTI_i;
  double CTATGAT_i;
  double CBBTBOT_i;
  double CBBTSTI_i;
  double CBBTGAT_i;
  double VBRBOT_i;
  double VBRSTI_i;
  double VBRGAT_i;
  double PBRBOT_i;
  double PBRSTI_i;
  double PBRGAT_i;
  double SWJUNEXP_i;
  double VJUNREF_i;
  double FJUNQ_i;
  double phitr;
  double phitd;
  double phitdinv;
  double perfc;
  double berfc;
  double cerfc;
  double ftdbot;
  double ftdsti;
  double ftdgat;
  double idsatbot;
  double idsatsti;
  double idsatgat;
  double vbibot;
  double vbisti;
  double vbigat;
  double vbiinvbot;
  double vbiinvsti;
  double vbiinvgat;
  double one_minus_PBOT;
  double one_minus_PSTI;
  double one_minus_PGAT;
  double one_over_one_minus_PBOT;
  double one_over_one_minus_PSTI;
  double one_over_one_minus_PGAT;
  double cjobot;
  double cjosti;
  double cjogat;
  double qprefbot;
  double qprefsti;
  double qprefgat;
  double qpref2bot;
  double qpref2sti;
  double qpref2gat;
  double wdepnulrbot;
  double wdepnulrsti;
  double wdepnulrgat;
  double wdepnulrinvbot;
  double wdepnulrinvsti;
  double wdepnulrinvgat;
  double VBIRBOTinv;
  double VBIRSTIinv;
  double VBIRGATinv;
  double atatbot;
  double atatsti;
  double atatgat;
  double btatpartbot;
  double btatpartsti;
  double btatpartgat;
  double fbbtbot;
  double fbbtsti;
  double fbbtgat;
  double fstopbot;
  double fstopsti;
  double fstopgat;
  double VBRinvbot;
  double VBRinvsti;
  double VBRinvgat;
  double slopebot;
  double slopesti;
  double slopegat;
  double VBIRBOTD_i;
  double VBIRSTID_i;
  double VBIRGATD_i;
  double PBOTD_i;
  double PSTID_i;
  double PGATD_i;
  double CSRHBOTD_i;
  double CSRHSTID_i;
  double CSRHGATD_i;
  double CTATBOTD_i;
  double CTATSTID_i;
  double CTATGATD_i;
  double CBBTBOTD_i;
  double CBBTSTID_i;
  double CBBTGATD_i;
  double VBRBOTD_i;
  double VBRSTID_i;
  double VBRGATD_i;
  double PBRBOTD_i;
  double PBRSTID_i;
  double PBRGATD_i;
  double VJUNREFD_i;
  double FJUNQD_i;
  double ftdbot_d;
  double ftdsti_d;
  double ftdgat_d;
  double idsatbot_d;
  double idsatsti_d;
  double idsatgat_d;
  double vbibot_d;
  double vbisti_d;
  double vbigat_d;
  double vbiinvbot_d;
  double vbiinvsti_d;
  double vbiinvgat_d;
  double one_minus_PBOT_d;
  double one_minus_PSTI_d;
  double one_minus_PGAT_d;
  double one_over_one_minus_PBOT_d;
  double one_over_one_minus_PSTI_d;
  double one_over_one_minus_PGAT_d;
  double cjobot_d;
  double cjosti_d;
  double cjogat_d;
  double qprefbot_d;
  double qprefsti_d;
  double qprefgat_d;
  double qpref2bot_d;
  double qpref2sti_d;
  double qpref2gat_d;
  double wdepnulrbot_d;
  double wdepnulrsti_d;
  double wdepnulrgat_d;
  double wdepnulrinvbot_d;
  double wdepnulrinvsti_d;
  double wdepnulrinvgat_d;
  double VBIRBOTinv_d;
  double VBIRSTIinv_d;
  double VBIRGATinv_d;
  double atatbot_d;
  double atatsti_d;
  double atatgat_d;
  double btatpartbot_d;
  double btatpartsti_d;
  double btatpartgat_d;
  double fbbtbot_d;
  double fbbtsti_d;
  double fbbtgat_d;
  double fstopbot_d;
  double fstopsti_d;
  double fstopgat_d;
  double VBRinvbot_d;
  double VBRinvsti_d;
  double VBRinvgat_d;
  double slopebot_d;
  double slopesti_d;
  double slopegat_d;
  // end verilog model variables=====
};

void registerDevice(const DeviceCountMap& deviceMap = DeviceCountMap(),
                    const std::set<int>& levelSet = std::set<int>());

} // namespace ADMSPSP102VA
} // namespace Device
} // namespace Xyce
#endif //Xyce_N_DEV_ADMSPSP102VA_h
